proc start_step { step } {
  set stopFile ".stop.rst"
  if {[file isfile .stop.rst]} {
    puts ""
    puts "*** Halting run - EA reset detected ***"
    puts ""
    puts ""
    return -code error
  }
  set beginFile ".$step.begin.rst"
  set platform "$::tcl_platform(platform)"
  set user "$::tcl_platform(user)"
  set pid [pid]
  set host ""
  if { [string equal $platform unix] } {
    if { [info exist ::env(HOSTNAME)] } {
      set host $::env(HOSTNAME)
    }
  } else {
    if { [info exist ::env(COMPUTERNAME)] } {
      set host $::env(COMPUTERNAME)
    }
  }
  set ch [open $beginFile w]
  puts $ch "<?xml version=\"1.0\"?>"
  puts $ch "<ProcessHandle Version=\"1\" Minor=\"0\">"
  puts $ch "    <Process Command=\".planAhead.\" Owner=\"$user\" Host=\"$host\" Pid=\"$pid\">"
  puts $ch "    </Process>"
  puts $ch "</ProcessHandle>"
  close $ch
}

proc end_step { step } {
  set endFile ".$step.end.rst"
  set ch [open $endFile w]
  close $ch
}

proc step_failed { step } {
  set endFile ".$step.error.rst"
  set ch [open $endFile w]
  close $ch
}

set_msg_config -id {Common-41} -limit 4294967295
set_msg_config -id {HDL 9-1061} -limit 100000
set_msg_config -id {HDL 9-1654} -limit 100000

start_step init_design
set rc [catch {
  create_msg_db init_design.pb
  set_param gui.test TreeTableDev
  set_property design_mode GateLvl [current_fileset]
  set_property webtalk.parent_dir D:/vproject/AMC13XG_T1/AMC13XG_T1.data/wt [current_project]
  set_property parent.project_dir D:/vproject/AMC13XG_T1 [current_project]
  add_files D:/vproject/AMC13XG_T1/AMC13XG_T1.runs/synth_1/AMC13_T1.dcp
  read_xdc D:/vproject/AMC13XG_T1/AMC13XG_T1.srcs/constrs_1/imports/AMC13XG_T1.srcs/amc13T1.xdc
  read_xdc D:/vproject/AMC13XG_T1/AMC13XG_T1.srcs/constrs_1/imports/AMC13XG_T1.srcs/ddr3_1_9a.xdc
  link_design -top AMC13_T1 -part xc7k325tffg900-2
  close_msg_db -file init_design.pb
} RESULT]
if {$rc} {
  step_failed init_design
  return -code error $RESULT
} else {
  end_step init_design
}

start_step opt_design
set rc [catch {
  create_msg_db opt_design.pb
  catch {write_debug_probes -quiet -force debug_nets}
  opt_design 
  write_checkpoint -force AMC13_T1_opt.dcp
  close_msg_db -file opt_design.pb
} RESULT]
if {$rc} {
  step_failed opt_design
  return -code error $RESULT
} else {
  end_step opt_design
}

start_step place_design
set rc [catch {
  create_msg_db place_design.pb
  place_design 
  catch { report_io -file AMC13_T1_io_placed.rpt }
  catch { report_clock_utilization -file AMC13_T1_clock_utilization_placed.rpt }
  catch { report_utilization -file AMC13_T1_utilization_placed.rpt -pb AMC13_T1_utilization_placed.pb }
  catch { report_control_sets -verbose -file AMC13_T1_control_sets_placed.rpt }
  write_checkpoint -force AMC13_T1_placed.dcp
  close_msg_db -file place_design.pb
} RESULT]
if {$rc} {
  step_failed place_design
  return -code error $RESULT
} else {
  end_step place_design
}

start_step route_design
set rc [catch {
  create_msg_db route_design.pb
  route_design 
  catch { report_drc -file AMC13_T1_drc_routed.rpt -pb AMC13_T1_drc_routed.pb }
  catch { report_power -file AMC13_T1_power_routed.rpt -pb AMC13_T1_power_summary_routed.pb }
  catch { report_route_status -file AMC13_T1_route_status.rpt -pb AMC13_T1_route_status.pb }
  catch { report_timing_summary -file AMC13_T1_timing_summary_routed.rpt -pb AMC13_T1_timing_summary_routed.pb }
  write_checkpoint -force AMC13_T1_routed.dcp
  close_msg_db -file route_design.pb
} RESULT]
if {$rc} {
  step_failed route_design
  return -code error $RESULT
} else {
  end_step route_design
}

start_step write_bitstream
set rc [catch {
  create_msg_db write_bitstream.pb
  write_bitstream -force AMC13_T1.bit 
  close_msg_db -file write_bitstream.pb
} RESULT]
if {$rc} {
  step_failed write_bitstream
  return -code error $RESULT
} else {
  end_step write_bitstream
}

