----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    15:55:15 07/09/2010 
-- Design Name: 
-- Module Name:    AMC_Link - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.std_logic_misc.all;
use work.amc13_pack.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;
Library UNIMACRO;
use UNIMACRO.vcomponents.all;

entity AMC_Link is
	generic(N : integer := 14; simulation : boolean := false);
	port(
    sysclk	                            : in   std_logic; -- 200MHz
    reset		                            : in   std_logic;
    resetCntr                           : in   std_logic;
    test		                            : in   std_logic;
    UsrClk                        			: in   std_logic;
    Ready			                 					: out  std_logic; -- Link to AMC established
    AMC_ID															: in  STD_LOGIC_VECTOR(3 downto 0);
    rxfsmresetdone                   		: in   std_logic;
    txfsmresetdone	                 		: in   std_logic;
    RxResetDone	                    		: in   std_logic;
    qpll_lock	                       		: in   std_logic;
    rxcommaalignen                 			: out  std_logic;
    DATA_VALID                     			: out  std_logic;
    RXDATA		                      		: in   std_logic_vector(15 downto 0);
    RXCHARISCOMMA                     	: in   std_logic_vector(1 downto 0);
    RXCHARISK		                      	: in   std_logic_vector(1 downto 0);
    RXNOTINTABLE                      	: in   std_logic_vector(1 downto 0);
    TXDATA     	                    		: out  std_logic_vector(15 downto 0);
    TXCHARISK                        		: out  std_logic_vector(1 downto 0);
-------------
    EventInfo                         	: out  std_logic_vector(19 downto 0);
    EventInfo_dav               				: out  std_logic;
    AMC_en			                   			: in  std_logic;
    AMC_DATA_RdEn                       : in   std_logic; -- enable reading AMC event
    EventInfoRdDone                     : in   std_logic; -- end of reading AMC event
    AMC_DATA                         		: out  std_logic_vector(63 downto 0);
    L1A_DATA		                      	: in   std_logic_vector(15 downto 0);
    L1A_WrEn                           	: in   std_logic;
    fake_header                        	: in   std_logic;
    fake_DATA		                      	: in   std_logic_vector(15 downto 0);
--    fake_evn		                      	: in   std_logic_vector(3 downto 0);
    fake_WrEn                          	: in   std_logic;
    fake_accept                        	: in   std_logic;
    bad_AMC			                   			: out  std_logic;
    AMC_OK			                   			: out  std_logic;
-- scan counters
    Cntr_RdEn                          	: in   std_logic;
    Cntr_str                          	: in   std_logic;
    Cntr_DATA		                      	: out   std_logic_vector(15 downto 0);
-- ipbus read signals
    ipb_addr	                      		: in   std_logic_vector(15 downto 0);
    ipb_rdata                      			: out  std_logic_vector(31 downto 0);
    debug_out                        		: out  std_logic_vector(255 downto 0);
-- TTC & TTS signals
    TTCclk	                            : in   std_logic;
    BC0			                   					: in  std_logic; -- AMC13 BC0 delayed by four TTC_clk cycles
    bcnt	                      		: in   std_logic_vector(3 downto 0);
    TTC_LOS                        			: in   std_logic;
    TTC_status	                      	: out   std_logic_vector(127 downto 0);
    TrigData                          	: out  std_logic_vector(7 downto 0);
    TTS			                          	: out  std_logic_vector(3 downto 0)
		);
end AMC_Link;

architecture Behavioral of AMC_Link is
COMPONENT HammingDecode
	PORT(
		clk : IN std_logic;
		din_valid : IN std_logic;
		din : IN std_logic_vector(23 downto 0);          
		dout_valid : OUT std_logic;
		dout : OUT std_logic_vector(17 downto 0);
		sgl_err : OUT std_logic;
		dbl_err : OUT std_logic
		);
END COMPONENT;
COMPONENT crc16D16
	PORT(
		clk : IN std_logic;
		init_crc : IN std_logic;
		we_crc : IN std_logic;
		d : IN std_logic_vector(15 downto 0);          
		crc : OUT std_logic_vector(15 downto 0)
		);
END COMPONENT;
COMPONENT crc16D16B
	PORT(
		clk : IN std_logic;
		init : IN std_logic;
		save : IN std_logic;
		restore : IN std_logic;
		we : IN std_logic;
		d : IN std_logic_vector(15 downto 0);          
		crc : OUT std_logic_vector(15 downto 0)
		);
END COMPONENT;
constant Acknowledge : std_logic_vector(7 downto 0) := x"12";
constant data : std_logic_vector(7 downto 0) := x"34";
constant InitRqst : std_logic_vector(7 downto 0) := x"56";
constant Counter : std_logic_vector(7 downto 0) := x"78";
constant K_word : std_logic_vector(15 downto 0) := x"3cbc"; -- sequence K28.5 K28.1
constant R_word : std_logic_vector(15 downto 0) := x"dcfb"; -- sequence K27.7 K28.6
constant eof_word : std_logic_vector(15 downto 0) := x"5cf7"; -- sequence K23.7 K28.2
constant IDLE : std_logic_vector(3 downto 0) := x"0"; -- TxState
constant SendK : std_logic_vector(3 downto 0) := x"1"; -- TxState sending comma
constant SendType : std_logic_vector(3 downto 0) := x"2"; -- TxState sending event data words
constant SendSEQ : std_logic_vector(3 downto 0) := x"3"; -- TxState sending sequence number
constant SendWC : std_logic_vector(3 downto 0) := x"4"; -- TxState sending payload word count
constant WaitCRC : std_logic_vector(3 downto 0) := x"5"; -- TxState same as IDLE
constant SendCRC : std_logic_vector(3 downto 0) := x"6"; -- TxState sending CRC
constant SendData : std_logic_vector(3 downto 0) := x"7"; -- TxState sending event data words
signal AMCRdy : std_logic := '0';
signal TxState: std_logic_vector(3 downto 0) := (others => '0');
signal InitLink : std_logic := '0';
signal RxResetDoneSyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal RXDATA_q : std_logic_vector(15 downto 0) := (others => '0');
signal TXDATA_i : std_logic_vector(15 downto 0) := (others => '0');
signal reset_SyncRegs : std_logic_vector(3 downto 0) := (others => '0');
signal fake_evn : std_logic_vector(3 downto 0) := (others => '0');
signal sgl_err : std_logic := '0';
signal dbl_err : std_logic := '0';
signal sel_TTC : std_logic := '0';
signal TTC_init : std_logic := '0';
signal TTC_FIFO_ra_FWD : std_logic := '0';
signal TTC_FIFO_ra_BWD : std_logic := '0';
signal BC0_match : std_logic := '0';
signal BC0_late : std_logic := '0';
signal got_AMC_BC0 : std_logic := '0';
signal TTC_lock : std_logic := '0';
signal TTC_lock_q : std_logic := '0';
signal TTC_lock_dl : std_logic := '0';
signal BC0_lock : std_logic := '0';
--signal bcnt : std_logic_vector(3 downto 0) := (others => '0');
signal MmCntr : std_logic_vector(3 downto 0) := (others => '0');
signal lock_cntr : std_logic_vector(8 downto 0) := (others => '0');
signal ec_delta : std_logic := '0';
signal delta_BC0 : std_logic_vector(5 downto 0) := (others => '0');
signal BC0_delay : std_logic_vector(3 downto 0) := (others => '0');
signal OldBC0_delay : std_logic_vector(3 downto 0) := (others => '0');
signal TTC_DataValid : std_logic := '0';
signal TTC_Data : std_logic_vector(23 downto 0) := (others => '0');
signal TTC_FIFO_we : std_logic := '0';
signal HammingOut : std_logic_vector(17 downto 0) := (others => '0');
signal TTC_FIFO_Di : std_logic_vector(12 downto 0) := (others => '0');
signal TTC_FIFO_Do : std_logic_vector(12 downto 0) := (others => '0');
signal TTC_FIFO_wa : std_logic_vector(3 downto 0) := (others => '0');
signal TTC_FIFO_wa_g : std_logic_vector(3 downto 0) := (others => '0');
signal TTC_FIFO_wa0_g_SyncRegs : std_logic_vector(1 downto 0) := (others => '0');
signal TTC_FIFO_wa1_g_SyncRegs : std_logic_vector(1 downto 0) := (others => '0');
signal TTC_FIFO_wa2_g_SyncRegs : std_logic_vector(1 downto 0) := (others => '0');
signal TTC_FIFO_wa3_g_SyncRegs : std_logic_vector(1 downto 0) := (others => '0');
signal TTC_FIFO_wap : std_logic_vector(3 downto 0) := (others => '0');
signal TTC_FIFO_ra : std_logic_vector(3 downto 0) := (others => '0');
signal check_packet : std_logic := '0';
signal ACK : std_logic := '0';
signal Abort : std_logic := '0';
signal bad_K : std_logic := '0';
signal SEQ_OK : std_logic := '0';
signal CRC_OK : std_logic := '0';
signal frame_OK : std_logic := '0';
signal ACK_OK : std_logic := '0';
signal WC_OKp : std_logic := '0';
signal WC_OK : std_logic := '0';
signal TypeInit : std_logic := '0';
signal TypeACK : std_logic := '0';
signal TypeData : std_logic := '0';
signal TypeCntr : std_logic := '0';
signal Receiving : std_logic := '0';
signal Receiving_q : std_logic := '0';
signal Header2 : std_logic := '0';
signal IsACK : std_logic := '0';
signal ACKNUM_full : std_logic := '0';
signal ACKNUM_empty : std_logic := '0';
signal ACKNUM_IN : std_logic_vector(7 downto 0) := (others => '0');
signal RxSEQNUM : std_logic_vector(7 downto 0) :=(others => '0');
signal SEQNUM : std_logic_vector(7 downto 0) :=(others => '0');
signal NextSEQNUM : std_logic_vector(7 downto 0) :=(others => '0');
signal ACKNUM : std_logic_vector(7 downto 0) := (others => '0');
signal CntrACKNUM : std_logic_vector(7 downto 0) := (others => '0');
signal ACKNUM_MUX : std_logic_vector(7 downto 0) := (others => '0');
signal ACKNUM_l : std_logic_vector(7 downto 0) := (others => '0');
signal ACKNUM_a : std_logic_vector(1 downto 0) := (others => '1');
signal RxType : std_logic_vector(15 downto 0) := (others => '0');
signal RxWC : std_logic_vector(11 downto 0) := (others => '0');
signal accept : std_logic := '0';
signal CntrAccept : std_logic := '0';
signal we_ACKNUM : std_logic := '0';
signal save_start_addr : std_logic := '0';
signal eof : std_logic := '0';
signal got_eof : std_logic := '0';
signal padding : std_logic := '0';
signal padding_dl : std_logic := '0';
signal pad_cntr : std_logic_vector(1 downto 0) := (others => '0');
signal dl_cntr : std_logic_vector(2 downto 0) := (others => '0');
signal we_rfifo : std_logic := '0';
signal rfifo : std_logic_vector(15 downto 0) := (others => '0');
signal rfifo_a : std_logic_vector(1 downto 0) := (others => '1');
signal EventBuf_ovf : std_logic := '0';
signal EventBuf_ovf_flag : std_logic := '0';
signal EventBuf_full : std_logic := '0';
signal ec_EventBuf_ra : std_logic := '0';
signal evn_word : std_logic := '0';
signal get_evn : std_logic := '0';
signal evn_OK : std_logic_vector(2 downto 0) := (others => '0');
signal evn : std_logic_vector(7 downto 0) := (others => '0');
signal InitAMCCRC : std_logic := '0';
signal AMCCRC : std_logic_vector(15 downto 0) := (others => '0');
signal EventBuf_we : std_logic_vector(0 downto 0) := (others => '0');
signal EventBuf_start : std_logic_vector(13 downto 0) := (others => '0');
signal EventBuf_wa : std_logic_vector(13 downto 0) := (others => '0');
signal EventBuf_wap : std_logic_vector(11 downto 0) := (others => '0');
signal EventBuf_wa1_SyncRegs : std_logic_vector(3 downto 0) := (others => '0');
signal EventBuf_ra : std_logic_vector(11 downto 0) := (others => '0');
signal EventBuf_ra1_SyncRegs : std_logic_vector(3 downto 0) := (others => '0');
signal EventBuf_rap : std_logic_vector(9 downto 0) := (others => '0');
signal EventBuf_space : std_logic_vector(9 downto 0) := (others => '0');
signal EventBuf_Di : std_logic_vector(15 downto 0) := (others => '0');
signal EventBuf_Do : std_logic_vector(63 downto 0) := (others => '0');
signal EventBufDoVld : std_logic := '0';
signal EventBufferRdEn : std_logic := '0';
signal EventBufferREGCE : std_logic := '0';
signal AMC_DATA_vld : std_logic := '0';
signal NewEvent : std_logic := '0';
signal EventInfoDi : std_logic_vector(19 downto 0) := (others => '0');
signal EventInfoDo : std_logic_vector(19 downto 0) := (others => '0');
signal EventInfo_a : std_logic_vector(3 downto 0) := (others => '1');
signal EventInfoToggleSyncRegs : std_logic_vector(3 downto 0) := (others => '0');
signal BOE : std_logic_vector(11 downto 0) := (others => '0');
signal EventInfo_full : std_logic := '0';
signal EventInfo_dav_i : std_logic := '0';
signal we_EventInfo : std_logic := '0';
signal EventInfoToggle : std_logic := '0';
signal re_EventInfo : std_logic := '0';
signal EventInfoRdDoneToggle : std_logic := '0';
signal EventInfoRdDoneToggleSyncRegs : std_logic_vector(3 downto 0) := (others => '0');
signal we_RxCRC : std_logic := '0';
signal Init_RxCRC : std_logic := '0';
signal RxCRC : std_logic_vector(15 downto 0) := (others => '0');
--type RAM1024X16 is array(0 to 1023) of std_logic_vector(15 downto 0);
--signal L1Ainfo : RAM1024X16;
signal L1Ainfo_we : std_logic_vector(0 downto 0) := (others => '0');
signal L1Ainfo_wa : std_logic_vector(9 downto 0) := (others => '0');
signal L1Ainfo_ra : std_logic_vector(9 downto 0) := (others => '0');
signal L1Ainfo_start : std_logic_vector(7 downto 0) := (others => '0');
signal L1AinfoDo : std_logic_vector(15 downto 0) := (others => '0');
signal L1Ainfo_wa2SyncRegs : std_logic_vector(3 downto 0) := (others => '0');
signal L1Ainfo_wap : std_logic_vector(7 downto 0) := (others => '0');
signal ec_L1Ainfo_ra : std_logic := '0';
signal L1Ainfo_empty : std_logic := '0';
signal L1AinfoSent : std_logic := '0';
signal AMC_L1A_full : std_logic := '0';
signal AMC_L1A : std_logic_vector(5 downto 0) := (others => '0');
signal timer : std_logic_vector(N downto 0) := (others => '0');
signal ReSend : std_logic := '0';
signal ReSend_l : std_logic := '0';
signal ReSendQueIn : std_logic_vector(15 downto 0) := (others => '0');
signal ReSendQueOut : std_logic_vector(15 downto 0) := (others => '0');
signal we_ReSendQue : std_logic := '0';
signal ReSendQue_a : std_logic_vector(1 downto 0) := (others => '1');
signal ReSendQue_empty : std_logic := '0';
signal ReSendQue_full : std_logic := '0';
signal got_comma : std_logic := '0';
signal GotCntr : std_logic := '0';
signal L1Asent : std_logic := '0';
signal TxType : std_logic_vector(7 downto 0) := (others => '0');
signal packet_wc : std_logic_vector(1 downto 0) := (others => '0');
signal we_TxCRC : std_logic := '0';
signal Init_TxCRC : std_logic := '0';
signal TxCRC : std_logic_vector(15 downto 0) := (others => '0');
signal TxIsK : std_logic := '0';
--signal RXNOTINTABLE : std_logic_vector(1 downto 0) := (others => '0');
signal we_CntrBuf : std_logic := '0';
signal CntrBuf_Di : std_logic_vector(15 downto 0) := (others => '0');
signal CntrBuf_Do : std_logic_vector(15 downto 0) := (others => '0');
signal CntrBuf_wa : std_logic_vector(5 downto 0) := (others => '0');
signal CntrBuf_ra5 : std_logic := '0';
signal we_CntrBuf2p : std_logic := '0';
signal we_CntrBuf2 : std_logic := '0';
signal CntrBuf2_Di : std_logic_vector(15 downto 0) := (others => '0');
signal CntrBuf2_SPO : std_logic_vector(15 downto 0) := (others => '0');
signal CntrBuf2_SPO_q : std_logic_vector(15 downto 0) := (others => '0');
signal CntrBuf2_Do : std_logic_vector(15 downto 0) := (others => '0');
signal CntrBuf2_wa : std_logic_vector(4 downto 0) := (others => '0');
signal CntrBuf2_ra4 : std_logic := '0';
signal BcntMmToggle : std_logic := '0';
signal BcntMmToggle_SyncRegs : std_logic_vector(3 downto 0) := (others => '0');
signal resetCntrCycle : std_logic := '0';
signal CntrBuf_valid : std_logic := '0';
signal cntrs : std_logic_vector(6 downto 0) := (others => '0');
signal sglErrCntr : std_logic_vector(6 downto 0) := (others => '0');
signal dblErrCntr : std_logic_vector(6 downto 0) := (others => '0');
signal BC0mmCntr : std_logic_vector(6 downto 0) := (others => '0');
signal BcntMmCntr : std_logic_vector(6 downto 0) := (others => '0');
signal ResendCntr : std_logic_vector(6 downto 0) := (others => '0');
signal AcceptCntr : std_logic_vector(6 downto 0) := (others => '0');
signal CntrAcceptCntr : std_logic_vector(6 downto 0) := (others => '0');
signal TotalWordCntr : std_logic_vector(15 downto 0) := (others => '0');
signal ACKcntr : std_logic_vector(6 downto 0) := (others => '0');
signal AbortCntr : std_logic_vector(6 downto 0) := (others => '0');
signal RxEventCntr : std_logic_vector(6 downto 0) := (others => '0');
signal RdEventCntr : std_logic_vector(6 downto 0) := (others => '0');
signal DataAbortCntr : std_logic_vector(6 downto 0) := (others => '0');
signal CntrAbortCntr : std_logic_vector(6 downto 0) := (others => '0');
signal BUSYCntr : std_logic_vector(6 downto 0) := (others => '0');
signal EvtEVNmmCntr : std_logic_vector(6 downto 0) := (others => '0');
signal EvtBCNmmCntr : std_logic_vector(6 downto 0) := (others => '0');
signal EvtOCNmmCntr : std_logic_vector(6 downto 0) := (others => '0');
signal ACKNUM_fullAbortCntr : std_logic_vector(6 downto 0) := (others => '0');
signal EventInfo_fullAbortCntr : std_logic_vector(6 downto 0) := (others => '0');
signal EventBuf_fullAbortCntr : std_logic_vector(6 downto 0) := (others => '0');
signal SEQAbortCntr : std_logic_vector(6 downto 0) := (others => '0');
signal CRCAbortCntr : std_logic_vector(6 downto 0) := (others => '0');
signal frameAbortCntr : std_logic_vector(6 downto 0) := (others => '0');
signal bad_KAbortCntr : std_logic_vector(6 downto 0) := (others => '0');
signal Cntr_ra : std_logic_vector(5 downto 0) := (others => '0');
signal TTS_wa : std_logic_vector(1 downto 0) := (others =>'0');
--signal TTS_ra : std_logic_vector(1 downto 0) := (others =>'0');
signal TTS_wa0_SyncRegs : std_logic_vector(1 downto 0) := (others =>'0');
signal TTS_wa1_SyncRegs : std_logic_vector(1 downto 0) := (others =>'0');
begin
debug_out <= (others => '0');
rxcommaalignen <= RxResetDoneSyncRegs(2);
TXDATA <= TXDATA_i;
EventInfo_dav <= EventInfo_dav_i;
AMC_OK <= AMCRdy;
TTC_status(0) <= BC0_lock;
TTC_status(1) <= TTC_lock;
TTC_status(84 downto 81) <= TTC_FIFO_Di(3 downto 0);
TTC_status(14 downto 6) <= TTC_FIFO_Di(12 downto 4);
TTC_status(18 downto 15) <= TTC_FIFO_wa;
TTC_status(31 downto 19) <= TTC_FIFO_Do;
TTC_status(35 downto 32) <= TTC_FIFO_ra;
TTC_status(53 downto 36) <= HammingOut when TTC_FIFO_we = '1' else (others => '0');
TTC_status(54) <= BC0_match;
TTC_status(57 downto 55) <= delta_BC0(2 downto 0);
TTC_status(58) <= BC0_late;
TTC_status(62 downto 59) <= BC0_delay;
TTC_status(66 downto 63) <= OldBC0_delay;
TTC_status(67) <= got_AMC_BC0;
TTC_status(68) <= TTC_lock;
TTC_status(69) <= BC0_lock;
TTC_status(78 downto 70) <= lock_cntr;
TTC_status(79) <= TTC_lock_dl;
TTC_status(80) <= TTC_init;
TTC_status(85) <= TTC_FIFO_ra_FWD;
TTC_status(86) <= TTC_FIFO_ra_BWD;
TTC_status(90 downto 87) <= MmCntr;
TTC_status(91) <= TTC_DataValid;
TTC_status(115 downto 92) <= TTC_Data;
TTC_status(116) <= TTC_FIFO_we;
TTC_status(117) <= sgl_err;
TTC_status(118) <= dbl_err;
TTC_status(127 downto 119) <= (others => '0');
i_HammingDecode: HammingDecode PORT MAP(
		clk => UsrClk,
		din_valid => TTC_DataValid,
		din => TTC_Data,
		dout_valid => TTC_FIFO_we,
		dout => HammingOut,
		sgl_err => sgl_err,
		dbl_err => dbl_err
	);
g_TTS : for i in 0 to 3 generate
   i_TTS : RAM32X1D
   port map (
      DPO => TTS(i),     -- Read-only 1-bit data output
      A0 => TTS_wa(0),       -- R/W address[0] input bit
      A1 => TTS_wa(1),       -- R/W address[1] input bit
      A2 => '0',       -- R/W address[2] input bit
      A3 => '0',       -- R/W address[3] input bit
      A4 => '0',       -- R/W address[4] input bit
      D => HammingOut(i+8),         -- Write 1-bit data input
      DPRA0 => TTS_wa1_SyncRegs(1), -- Read-only address[0] input bit
      DPRA1 => TTS_wa0_SyncRegs(1), -- Read-only address[1] input bit
      DPRA2 => '0', -- Read-only address[2] input bit
      DPRA3 => '0', -- Read-only address[3] input bit
      DPRA4 => '0', -- Read-only address[4] input bit
      WCLK => UsrClk,   -- Write clock input
      WE => TTC_FIFO_we       -- Write enable input
   );
end generate;
process(UsrClk)
begin
	if(UsrClk'event and UsrClk = '1')then
		if(TTC_FIFO_we = '1')then
			TTS_wa(1) <= TTS_wa(0);
			TTS_wa(0) <= not TTS_wa(1);
		end if;
	end if;
end process;
process(sysclk)
begin
	if(sysclk'event and sysclk = '1')then
		TTS_wa0_SyncRegs <= not TTS_wa0_SyncRegs(0) & TTS_wa(0);
		TTS_wa1_SyncRegs <= TTS_wa1_SyncRegs(0) & TTS_wa(1);
--		TTS_ra(1) <= not TTS_wa_SyncRegs(0);
--		TTS_ra(0) <= TTS_wa_SyncRegs(1);
	end if;
end process;
process(UsrClk,BC0_lock)
begin
	if(BC0_lock = '0')then
		BcntMmToggle_SyncRegs <= (others => '0');
		BcntMmCntr <= (others => '0');
	elsif(UsrClk'event and UsrClk = '1')then
		if(resetCntr = '1')then
			BcntMmToggle_SyncRegs <= (others => '0');
		else
			BcntMmToggle_SyncRegs <= BcntMmToggle_SyncRegs(2 downto 0) & BcntMmToggle;
		end if;
		if(resetCntr = '1')then
			BcntMmCntr <= (others => '0');
		elsif(BcntMmToggle_SyncRegs(3) /= BcntMmToggle_SyncRegs(2))then
			BcntMmCntr <= BcntMmCntr + 1;
		end if;
	end if;
end process;
process(UsrClk)
begin
	if(UsrClk'event and UsrClk = '1')then
    	if(resetCntr = '1')then
            BC0mmCntr <= (others => '0');
        elsif(dbl_err = '0' and TTC_FIFO_we = '1' and HammingOut(17) /= HammingOut(16))then
            BC0mmCntr <= BC0mmCntr + 1;
        end if;
		if(Cntr_RdEn = '0')then
			Cntr_ra <= (others => '0');
		elsif(Cntr_str = '1')then
			Cntr_ra <= Cntr_ra + 1;
		end if;
		if(AMC_en = '0' or Cntr_RdEn = '0' or (Cntr_ra(5) = '0' and CntrBuf_valid = '0'))then
			Cntr_DATA <= (others => '0');
		elsif(Cntr_ra(5) = '0')then
			Cntr_DATA <= CntrBuf_Do;
		elsif(or_reduce(Cntr_ra(4 downto 0)) = '0')then
			Cntr_DATA <= TotalWordCntr;
		else
			Cntr_DATA <= CntrBuf2_Do;
		end if;
		if(resetCntr = '1')then
			sglErrCntr <= (others => '0');
		elsif(sgl_err = '1')then
			sglErrCntr <= sglErrCntr + 1;
		end if;
		if(resetCntr = '1')then
			dblErrCntr <= (others => '0');
		elsif(dbl_err = '1')then
			dblErrCntr <= dblErrCntr + 1;
		end if;
		if(resetCntr = '1')then
			ResendCntr <= (others => '0');
		elsif(Resend = '1')then
			ResendCntr <= ResendCntr + 1;
		end if;
		if(resetCntr = '1')then
			AcceptCntr <= (others => '0');
		elsif(accept = '1')then
			AcceptCntr <= AcceptCntr + 1;
		end if;
		if(resetCntr = '1')then
			CntrAcceptCntr <= (others => '0');
		elsif(CntrAccept = '1')then
			CntrAcceptCntr <= CntrAcceptCntr + 1;
		end if;
		if(resetCntr = '1')then
			ACKcntr <= (others => '0');
		elsif(ACK = '1')then
			ACKcntr <= ACKcntr + 1;
		end if;
		if(resetCntr = '1')then
			TotalWordcntr <= (others => '0');
		elsif(we_EventInfo = '1')then
			TotalWordcntr <= TotalWordcntr + (x"0" & EventInfoDi(11 downto 0));
		end if;
		if(resetCntr = '1')then
			RxEventCntr <= (others => '0');
		elsif(we_EventInfo = '1')then
			RxEventCntr <= RxEventCntr + 1;
		end if;
		if(resetCntr = '1')then
			RdEventCntr <= (others => '0');
		elsif(re_EventInfo = '1')then
			RdEventCntr <= RdEventCntr + 1;
		end if;
		if(resetCntr = '1')then
			DataAbortCntr <= (others => '0');
		elsif(Abort = '1' and TypeData = '1')then
			DataAbortCntr <= DataAbortCntr + 1;
		end if;
		if(resetCntr = '1')then
			CntrAbortCntr <= (others => '0');
		elsif(Abort = '1' and TypeCntr = '1')then
			CntrAbortCntr <= CntrAbortCntr + 1;
		end if;
		Abort <= check_packet and (TypeData or TypeCntr) and not(SEQ_OK and CRC_OK and frame_OK and not bad_K and not((ACKNUM_full or EventInfo_full or EventBuf_ovf) and TypeData));
		if(resetCntr = '1')then
			ACKNUM_fullAbortCntr <= (others => '0');
		elsif(Abort = '1' and ACKNUM_full = '1')then
			ACKNUM_fullAbortCntr <= ACKNUM_fullAbortCntr + 1;
		end if;
		if(resetCntr = '1')then
			EventInfo_fullAbortCntr <= (others => '0');
		elsif(Abort = '1' and EventInfo_full = '1')then
			EventInfo_fullAbortCntr <= EventInfo_fullAbortCntr + 1;
		end if;
		if(resetCntr = '1')then
			EventBuf_fullAbortCntr <= (others => '0');
		elsif(Abort = '1' and EventBuf_full = '1')then
			EventBuf_fullAbortCntr <= EventBuf_fullAbortCntr + 1;
		end if;
		if(resetCntr = '1')then
			SEQAbortCntr <= (others => '0');
		elsif(Abort = '1' and SEQ_OK = '0')then
			SEQAbortCntr <= SEQAbortCntr + 1;
		end if;
		if(resetCntr = '1')then
			CRCAbortCntr <= (others => '0');
		elsif(Abort = '1' and CRC_OK = '0')then
			CRCAbortCntr <= CRCAbortCntr + 1;
		end if;
		if(resetCntr = '1')then
			frameAbortCntr <= (others => '0');
		elsif(Abort = '1' and frame_OK = '0')then
			frameAbortCntr <= frameAbortCntr + 1;
		end if;
		if(resetCntr = '1')then
			bad_KAbortCntr <= (others => '0');
		elsif(Abort = '1' and bad_K = '1')then
			bad_KAbortCntr <= bad_KAbortCntr + 1;
		end if;
		if(resetCntr = '1' or test = '1' or AMC_en = '0')then
			EvtEVNmmCntr <= (others => '0');
		elsif(we_EventInfo = '1' and RxType(0) = '0')then
			EvtEVNmmCntr <= EvtEVNmmCntr + 1;
		end if;
		if(resetCntr = '1' or test = '1' or AMC_en = '0')then
			EvtBCNmmCntr <= (others => '0');
		elsif(we_EventInfo = '1' and RxType(2) = '0')then
			EvtBCNmmCntr <= EvtBCNmmCntr + 1;
		end if;
		if(resetCntr = '1' or test = '1' or AMC_en = '0')then
			EvtOCNmmCntr <= (others => '0');
		elsif(we_EventInfo = '1' and RxType(1) = '0')then
			EvtOCNmmCntr <= EvtOCNmmCntr + 1;
		end if;
		TTC_FIFO_wa_g(3) <= TTC_FIFO_wa(3);
		TTC_FIFO_wa_g(2) <= TTC_FIFO_wa(3) xor TTC_FIFO_wa(2);
		TTC_FIFO_wa_g(1) <= TTC_FIFO_wa(2) xor TTC_FIFO_wa(1);
		TTC_FIFO_wa_g(0) <= TTC_FIFO_wa(1) xor TTC_FIFO_wa(0);
	end if;
end process;
g_TTC_FIFO : for i in 0 to 12 generate
   i_TTC_FIFO : RAM32X1D
   port map (
      DPO => TTC_FIFO_Do(i),     -- Read-only 1-bit data output
      SPO => open,     -- R/W 1-bit data output
      A0 => TTC_FIFO_wa(0),       -- R/W address[0] input bit
      A1 => TTC_FIFO_wa(1),       -- R/W address[1] input bit
      A2 => TTC_FIFO_wa(2),       -- R/W address[2] input bit
      A3 => TTC_FIFO_wa(3),       -- R/W address[3] input bit
      A4 => '1',       -- R/W address[4] input bit
      D => TTC_FIFO_Di(i),         -- Write 1-bit data input
      DPRA0 => TTC_FIFO_ra(0), -- Read-only address[0] input bit
      DPRA1 => TTC_FIFO_ra(1), -- Read-only address[1] input bit
      DPRA2 => TTC_FIFO_ra(2), -- Read-only address[2] input bit
      DPRA3 => TTC_FIFO_ra(3), -- Read-only address[3] input bit
      DPRA4 => AMC_en, -- Read-only address[4] input bit
      WCLK => UsrClk,   -- Write clock input
      WE => TTC_FIFO_we       -- Write enable input
   );
end generate;
TTC_FIFO_Di <= HammingOut(17) & HammingOut(15 downto 12) & HammingOut(7 downto 0);
TrigData <= TTC_FIFO_Do(7 downto 0);
--TTS <= TTC_FIFO_Do(11 downto 8);
process(TTCclk, bcnt, TTC_FIFO_Do)
variable delta : std_logic_vector(3 downto 0);
begin
delta := bcnt - TTC_FIFO_Do(11 downto 8);
	if(TTCclk'event and TTCclk = '1')then
		TTC_FIFO_wa0_g_SyncRegs <= TTC_FIFO_wa0_g_SyncRegs(0) & TTC_FIFO_wa_g(0);
		TTC_FIFO_wa1_g_SyncRegs <= TTC_FIFO_wa1_g_SyncRegs(0) & TTC_FIFO_wa_g(1);
		TTC_FIFO_wa2_g_SyncRegs <= TTC_FIFO_wa2_g_SyncRegs(0) & TTC_FIFO_wa_g(2);
		TTC_FIFO_wa3_g_SyncRegs <= TTC_FIFO_wa3_g_SyncRegs(0) & TTC_FIFO_wa_g(3);
		TTC_FIFO_wap(3) <= TTC_FIFO_wa3_g_SyncRegs(1);
		TTC_FIFO_wap(2) <= TTC_FIFO_wa3_g_SyncRegs(1) xor TTC_FIFO_wa2_g_SyncRegs(1);
		TTC_FIFO_wap(1) <= TTC_FIFO_wa3_g_SyncRegs(1) xor TTC_FIFO_wa2_g_SyncRegs(1) xor TTC_FIFO_wa1_g_SyncRegs(1);
		TTC_FIFO_wap(0) <= TTC_FIFO_wa3_g_SyncRegs(1) xor TTC_FIFO_wa2_g_SyncRegs(1) xor TTC_FIFO_wa1_g_SyncRegs(1) xor TTC_FIFO_wa0_g_SyncRegs(1);
		if(TTC_FIFO_ra = x"3")then
		  TTC_status(5 downto 2) <= TTC_FIFO_wap;
		end if;
		if(ec_delta = '0')then
		  delta_BC0 <= (others => '0');
		else
		  delta_BC0 <= delta_BC0 + 1;
		end if;
		if(delta_BC0(5) = '1')then
		  ec_delta <= '0';
		elsif(Bc0 /= TTC_FIFO_Do(12))then
		  ec_delta <= not ec_delta;
		end if;
		if(ec_delta = '0')then
		  if(BC0 = '1')then
			BC0_late <= '0';
		  elsif(TTC_FIFO_Do(12) = '1')then
			BC0_late <= '1';
		  end if;
		end if;
		if(BC0 = '1')then
			BC0_match <= TTC_FIFO_Do(12);
			OldBC0_delay <= BC0_delay;
			got_AMC_BC0 <= TTC_FIFO_Do(12);
		elsif(BC0_match = '0')then
--			if(and_reduce(delta_BC0) = '1')then
--				BC0_late <= '1';
--			end if;
			if(TTC_FIFO_Do(12) = '1')then
				BC0_delay <= BC0_late & delta_BC0(2 downto 0);
				got_AMC_BC0 <= '1';
			end if;
		end if;
		if(TTC_lock = '1')then
			lock_cntr <= (others => '0');
		elsif(BC0 = '1')then
			if(got_AMC_BC0 = '1' and BC0_delay = OldBC0_delay)then
				lock_cntr <= lock_cntr + 1;
			else
				lock_cntr <= (others => '0');
			end if;
--			TTC_status(84 downto 81) <= BC0_delay;
		end if;
		TTC_lock_q <= TTC_lock;
		TTC_init <= TTC_lock and not TTC_lock_q;
--		if(BC0 = '1')then
--			bcnt <= x"0";
--		else
--			bcnt <= bcnt + 1;
--		end if;
		if(TTC_lock = '0')then
			TTC_lock_dl <= '0';
		elsif(BC0 = '1')then
			TTC_lock_dl <= '1';
		end if;
--		if(TTC_lock_dl = '0' or BC0 = '1')then
		if(BC0_lock = '0' or BC0 = '1')then
			MmCntr <= (others => '0');
		elsif(bcnt /= TTC_FIFO_Do(11 downto 8))then
			MmCntr <= MmCntr + 1;
		end if;
--		if(TTC_lock = '1' and bcnt /= TTC_FIFO_Do(15 downto 12) and TTC_FIFO_ra_FWD = '0' and TTC_FIFO_ra_BWD = '0')then
		if(TTC_lock = '1' and bcnt /= TTC_FIFO_Do(11 downto 8) and TTC_FIFO_ra_FWD = '0' and TTC_FIFO_ra_BWD = '0')then
			if(BC0_lock = '1')then
				if(delta(3) = '1')then
					TTC_FIFO_ra_FWD <= '1';
				else
					TTC_FIFO_ra_BWD <= '1';
				end if;
			else
				if(BC0_late = '0')then
					TTC_FIFO_ra_FWD <= '1';
				else
					TTC_FIFO_ra_BWD <= '1';
				end if;
			end if;
		else
			TTC_FIFO_ra_FWD <= '0';
			TTC_FIFO_ra_BWD <= '0';
		end if;
		if(TTC_lock = '0')then
			BC0_lock <= '0';
		elsif(BC0 = '1' and TTC_FIFO_Do(12) = '1' and BC0_match = '1')then
			BC0_lock <= '1';
		end if;
	end if;
end process;
process(TTCclk, reset, AMC_en)
begin
	if(reset = '1' or AMC_en = '0')then
		TTC_lock <= '0';
	elsif(TTCclk'event and TTCclk = '1')then
		if(MmCntr(3) = '1')then
			TTC_lock <= '0';
		elsif(simulation and lock_cntr(0) = '1')then
			TTC_lock <= '1';
		elsif(lock_cntr(8) = '1')then
			TTC_lock <= '1';
		end if;
	end if;
end process;
process(TTCclk, reset, TTC_init)
begin
	if(reset = '1' or TTC_init = '1')then
		TTC_FIFO_ra <= (others => '0');
	elsif(TTCclk'event and TTCclk = '1')then
		if(TTC_init = '1')then
			TTC_FIFO_ra <= (others => '0');
		elsif(TTC_FIFO_ra_BWD = '0')then
			TTC_FIFO_ra <= TTC_FIFO_ra + ("000" & TTC_FIFO_ra_FWD) + 1;
		end if;
	end if;
end process;
process(TTCclk, resetCntr)
begin
	if(resetCntr = '1')then
		BcntMmToggle <= '0';
	elsif(TTCclk'event and TTCclk = '1')then
		if(BC0_lock = '0')then
			BcntMmToggle <= '0';
		elsif(bcnt /= TTC_FIFO_Do(11 downto 8))then
			BcntMmToggle <= not BcntMmToggle;
		end if;
	end if;
end process;
process(UsrClk,reset,TTC_init)
begin
	if(reset = '1' or TTC_init = '1')then
		TTC_FIFO_wa <= x"8";
	elsif(UsrClk'event and UsrClk = '1')then
		if(TTC_FIFO_we = '1')then
			TTC_FIFO_wa <= TTC_FIFO_wa + 1;
		end if;
	end if;
end process;
process(UsrClk,RxResetDone)
begin
	if(RxResetDone = '0')then
		RxResetDoneSyncRegs <= (others => '0');
		AMCRdy <= '0';
	elsif(UsrClk'event and UsrClk = '1')then
		RxResetDoneSyncRegs <= RxResetDoneSyncRegs(1 downto 0) & '1';
--		if(rxfsmresetdone = '1' and txfsmresetdone = '1')then
		if(txfsmresetdone = '1')then
			AMCRdy <= '1';
		end if;
	end if;
end process;
process(UsrClk,reset,RxResetDone,txfsmresetdone,qpll_lock)
begin
	if(reset = '1' or RxResetDone = '0' or txfsmresetdone = '0' or qpll_lock = '0')then
		reset_SyncRegs <= (others => '1');
		EventInfoToggle <= '0';
	elsif(UsrClk'event and UsrClk = '1')then
		reset_SyncRegs <= reset_SyncRegs(2 downto 0) & '0';
		if((we_EventInfo = '1' and EventInfo_a = x"f") or (re_EventInfo = '1' and (EventInfo_a /= x"0" or we_EventInfo = '1')))then
			EventInfoToggle <= not EventInfoToggle;
		end if;
	end if;
end process;
process(UsrClk)
begin
	if(UsrClk'event and UsrClk = '1')then
--		if(AMC_en = '0' or test = '1')then
--			DATA_VALID <= '1';
		if(RxResetDoneSyncRegs(2) = '0' or or_reduce(RxNotInTable) = '1' or bad_K = '1')then
			DATA_VALID <= '0';
		elsif(RXCHARISK(0) = '1' and RXDATA(7 downto 0) = x"bc")then
			DATA_VALID <= '1';
		end if;
		if(reset_SyncRegs(3) = '1' or AMC_en = '0')then
			Ready <= '0';
		elsif(InitLink = '1' and ACK = '1')then
			Ready <= '1';
		end if;
	end if;
end process;
ReSend <= timer(N);
process(UsrClk)
begin
	if(UsrClk'event and UsrClk = '1')then
		if(timer(N) = '1' or ACK = '1' or (ReSendQue_empty = '1' and (got_comma = '0' or InitLink = '0')))then
			timer <= (others => '0');
		else
			timer <= timer + 1;
		end if;
		if(ReSend = '1')then
			ReSend_l <= '1';
		elsif(TxState = SendSEQ)then
			ReSend_l <= '0';
		end if;
		if(RXCHARISK = "01" and RXDATA(7 downto 0) = x"bc")then
			sel_TTC <= '1';
		elsif(RXCHARISK = "00")then
			sel_TTC <= '0';
		end if;
		if(RXCHARISK(1) = '0')then
			TTC_Data(23 downto 8) <= RXDATA;
			TTC_Data(7 downto 0) <= TTC_Data(23 downto 16);
		end if;
		if(sel_TTC = '1' and RXCHARISK = "00" and AMCRdy = '1')then
			TTC_DataValid <= '1';
		else
			TTC_DataValid <= '0';
		end if;
-- Comma ends a packet and after that, any D-word marks the beginning of a packet
		if(RXCHARISCOMMA  = "11" or test = '1' or AMCRdy = '0')then
			Receiving <= '0';
		elsif(sel_TTC = '0' and RXCHARISK = "00")then
			Receiving <= '1';
		end if;
		if(sel_TTC = '0')then
			if(RXCHARISK = "00")then
				RXDATA_q <= RXDATA;
				if(Receiving = '0')then
					bad_K <= '0';
					RxSEQNUM <= RXDATA(15 downto 8);
					if(RXDATA(7 downto 0) = InitRqst and InitLink = '1')then
						TypeInit <= '1';
					else
						TypeInit <= '0';
					end if;
					if(RXDATA(7 downto 0) = Acknowledge)then
						TypeACK <= '1';
					else
						TypeACK <= '0';
					end if;
					if(RXDATA(7 downto 0) = data)then
						TypeData <= '1';
					else
						TypeData <= '0';
					end if;
					if(RXDATA(7 downto 0) = Counter)then
						TypeCntr <= '1';
					else
						TypeCntr <= '0';
					end if;
					Header2 <= '1';
				else
					Header2 <= '0';
					evn_word <= Header2 and get_evn and TypeData;
					if(Header2 = '1')then
						RxWC <= (others => '0');
						RxType <= RXDATA;
					else
						RxWC <= RxWC + 1;
					end if;
					if(RxWC = RxData(11 downto 0))then
						WC_OKp <= '1';
					else
						WC_OKp <= '0';
					end if;
					WC_OK <= WC_OKp;
					if(evn = RxData(15 downto 8))then
						evn_OK(0) <= '1';
					else
						evn_OK(0) <= '0';
					end if;
					evn_OK(2 downto 1) <= evn_OK(1 downto 0);
				end if;
			elsif((RXCHARISK = "11" and RXDATA /= R_word and RXDATA /= eof_word and RXCHARISCOMMA /= "11") or (RXCHARISK = "01" and RXDATA(7 downto 0) /= x"bc"))then
				bad_K <= '1';
			end if;
		end if;
		if((TypeACK = '1' or TypeInit = '1') and RXType(15 downto 8) = ReSendQueOut(15 downto 8) and ReSendQue_empty = '0')then -- incoming acknowledge number is what waited for
			ACK_OK <= '1';
		else
			ACK_OK <= '0';
		end if;
		if(Receiving = '0')then
			ACKNUM_IN <= RxSEQNUM;
		end if;
		if(RxSEQNUM = NextSEQNUM)then -- incoming sequence number is what waited for 
			SEQ_OK <= '1';
		else
			SEQ_OK <= '0';
		end if;
		if(or_reduce(RxCRC) = '0')then
			CRC_OK <= '1';
		else
			CRC_OK <= '0';
		end if;
		if(WC_OK = '1' and (((TypeData = '1' or TypeCntr = '1' or TypeACK = '1') and InitLink = '0') or (TypeInit = '1' and InitLink = '1')))then
			frame_OK <= '1';
		else
			frame_OK <= '0';
		end if;
		if(RXCHARISCOMMA = "11" and Receiving = '1')then
			check_packet <= '1';
		else
			check_packet <= '0';
		end if;
		accept <= check_packet and SEQ_OK and CRC_OK and frame_OK and not EventInfo_full and not EventBuf_ovf and not bad_K and not ACKNUM_full and TypeData;
		CntrAccept <= check_packet and SEQ_OK and CRC_OK and frame_OK and not bad_K and TypeCntr;
		we_ACKNUM <= check_packet and CRC_OK and frame_OK and not bad_K and not ACKNUM_full and not EventInfo_full and not EventBuf_ovf and TypeData;
		ACK <= check_packet and ACK_OK and CRC_OK and frame_OK and not bad_K;
		Abort <= check_packet and (TypeData or TypeCntr) and not(SEQ_OK and CRC_OK and frame_OK and not bad_K and not((ACKNUM_full or EventInfo_full or EventBuf_ovf) and TypeData));
		if(InitLink = '1')then
			get_evn <= '1';
		elsif(accept = '1')then
			get_evn <= got_eof;
		end if;
		if(evn_word = '1')then
			evn <= RxData(7 downto 0);
		end if;
		if(test = '1')then
			EventInfoDi(19 downto 15) <= fake_evn & '1';
		elsif(accept = '1')then
			EventInfoDi(19 downto 15) <= evn(3 downto 0) & evn_OK(2);
		end if;
		if(TxState = SendCRC and TxType = Counter)then
			GotCntr <= '0';
		elsif(CntrAccept = '1')then
			GotCntr <= '1';
		end if;
		if(InitLink = '1')then
			CntrACKNUM <= x"00";
		elsif(CntrAccept = '1')then
			CntrACKNUM <= RxSEQNUM;
		end if;
		if(reset_SyncRegs(3) = '1')then
			ACKNUM_a <= (others => '1');
		elsif(we_ACKNUM = '1' and (TxState /= SendCRC or IsACK = '0'))then
			ACKNUM_a <= ACKNUM_a + 1;
		elsif(we_ACKNUM = '0' and TxState = SendCRC and IsACK = '1')then
			ACKNUM_a <= ACKNUM_a - 1;
		end if;
		if(ACKNUM_a = "11")then
			ACKNUM_empty <= '1';
		else
			ACKNUM_empty <= '0';
		end if;
		if(ACKNUM_a = "10")then
			ACKNUM_full <= '1';
		else
			ACKNUM_full <= '0';
		end if;
		if(InitLink = '1')then
			ACKNUM_l <= x"00";
		elsif(ACKNUM_a /= "11")then
			ACKNUM_l <= ACKNUM;
		end if;
		if(reset_SyncRegs(3) = '1')then
			NextSEQNUM <= x"00";
		elsif(accept = '1')then
			NextSEQNUM <= NextSEQNUM(6 downto 0) & not(NextSEQNUM(7) xor NextSEQNUM(5) xor NextSEQNUM(4) xor NextSEQNUM(3));
		end if;
-- Receiving of eof K-word in a packet marks the end of the event
		if(Receiving = '0' or InitLink = '1')then
			eof <= '0';
		elsif(RXCHARISK = "11" and RXDATA = eof_word)then
			eof <= '1';
		end if;
-- latches eof signal
		if(check_packet = '1')then
			got_eof <= eof;
		end if;
-- if buffer is getting full during packet receiving, buffer overflow error is set 
		if(Receiving = '0')then
			EventBuf_ovf <= '0';
		elsif(EventBuf_full = '1')then
			EventBuf_ovf <= '1';
		end if;
		Receiving_q <= Receiving;
-- this flag is sent back to LSC as information of LDC status
		if(Receiving = '0' and Receiving_q = '1')then
			EventBuf_ovf_flag <= EventBuf_ovf;
		end if;
-- if a packet is rejected, buffer write address is moved back to where it was at the start of the packet(discarding data from this packet)
		if(InitLink = '1')then
			EventBuf_wa <= (others => '0');
		elsif(abort = '1')then
			EventBuf_wa <= EventBuf_start;
		elsif(EventBuf_we = "1")then
			EventBuf_wa <= EventBuf_wa + 1;
		end if;
-- if a packet is accepted, packet starting write address is updated
		if(InitLink = '1')then
			EventBuf_start <= (others => '0');
		elsif(save_start_addr = '1')then
			EventBuf_start <= EventBuf_wa;
		end if;
-- data buffer has asynchronous clock at write/read sides, all pointers needs resync to cross between clock domains
		if(InitLink = '1')then
			EventBuf_ra1_SyncRegs <= (others => '0');
			EventBuf_rap <= (others => '0');
		else
			EventBuf_ra1_SyncRegs <= EventBuf_ra1_SyncRegs(2 downto 0) & EventBuf_ra(1);
			if(EventBuf_ra1_SyncRegs(3 downto 2) = "10")then
				EventBuf_rap <= EventBuf_rap + 1;
			end if;
		end if;
-- buffer room used for writing incoming packet data
		EventBuf_space <= EventBuf_wa(13 downto 4) - EventBuf_rap;
-- buffer almost full
		if(and_reduce(EventBuf_space(9 downto 1)) = '1')then
			EventBuf_full <= '1';
		elsif(RXCHARISCOMMA = "11")then
			EventBuf_full <= '0';
		end if;
	end if;
end process;
i_RxCRC: crc16D16 PORT MAP(
		clk => UsrClk,
		init_crc => Init_RxCRC,
		we_crc => we_RxCRC,
		d => RXDATA,
		crc => RxCRC
	);
we_RxCRC <= '1' when RXCHARISK = "00" and sel_TTC = '0' else '0';
Init_RxCRC <= '1' when RXCHARISCOMMA = "11" else '0';
i_AMCCRC: crc16D16B PORT MAP(
		clk => UsrClk,
		init => InitAMCCRC,
		save => accept,
		restore => abort,
		we => EventBuf_we(0),
		d => EventBuf_Di,
		crc => AMCCRC
	);
process(UsrClk)
begin
	if(UsrClk'event and UsrClk = '1')then
		if(InitLink = '1' or we_EventInfo = '1' or (abort = '1' and get_evn = '1'))then
			InitAMCCRC <= '1';
		elsif(EventBuf_we(0) = '1' and padding_dl = '0')then
			InitAMCCRC <= '0';
		end if;
		padding_dl <= padding;
	end if;
end process;
g_ACKNUM : for i in 0 to 7 generate
   i_ACKNUM : SRL16E
   port map (
      Q => ACKNUM(i),       -- SRL data output
      A0 => ACKNUM_a(0),     -- Select[0] input
      A1 => ACKNUM_a(1),     -- Select[1] input
      A2 => '0',     -- Select[2] input
      A3 => '0',     -- Select[3] input
      CE => we_ACKNUM,     -- Clock enable input
      CLK => UsrClk,   -- Clock input
      D => ACKNUM_IN(i)        -- SRL data input
   );
end generate;
-- this fifo is used to hold the last couple of received data words before writing to the data buffer.
-- this is because 1. word count and CRC of the packet should not be written to the buffer and more importantly,
-- if the packet is the last packet of the event, the last data word must be properly marked with the LinkCtrl
-- before being written to the buffer. 3. next packet data may arrive before a decision being made to accept or reject a packet.
g_rfifo: for i in 0 to 15 generate
   i_rfifo : SRL16E
   port map (
      Q => rfifo(i),       -- SRL data output
      A0 => '1',     -- Select[0] input
      A1 => '0',     -- Select[1] input
      A2 => '0',     -- Select[2] input
      A3 => '0',     -- Select[3] input
      CE => we_rfifo,     -- Clock enable input
      CLK => UsrClk,   -- Clock input
      D => RXDATA_q(i)        -- SRL data input
   );
end generate;
process(UsrClk)
begin
	if(UsrClk'event and UsrClk = '1')then
		if(sel_TTC = '0' and RXCHARISK = "00")then
			we_rfifo <= '1';
		else
			we_rfifo <= '0';
		end if;
		if(InitLink = '1' or check_packet = '1')then
			dl_cntr <= "000";
		elsif(we_rfifo = '1')then
			dl_cntr(2) <= dl_cntr(2) or (dl_cntr(1) and dl_cntr(0));
			dl_cntr(1) <= dl_cntr(1) xor dl_cntr(0);
			dl_cntr(0) <= not dl_cntr(2) and not dl_cntr(0);
		end if;
	end if;
end process;
g_EventBuffer : for i in 0 to 7 generate
	i_EventBuffer : BRAM_SDP_MACRO
   generic map (
      BRAM_SIZE => "36Kb", -- Target BRAM, "18Kb" or "36Kb" 
      DEVICE => "7SERIES", -- Target device: "VIRTEX5", "VIRTEX6", "7SERIES", "SPARTAN6" 
      WRITE_WIDTH => 2,    -- Valid values are 1-72 (37-72 only valid when BRAM_SIZE="36Kb")
      READ_WIDTH => 8,     -- Valid values are 1-72 (37-72 only valid when BRAM_SIZE="36Kb")
      DO_REG => 1, -- Optional output register (0 or 1)
      SIM_COLLISION_CHECK => "NONE") -- Collision check enable "ALL", "WARNING_ONLY", 
                                    -- "GENERATE_X_ONLY" or "NONE"       
   port map (
      DO => EventBuf_Do(i*8+7 downto i*8),         -- Output read data port, width defined by READ_WIDTH parameter
      DI => EventBuf_Di(i*2+1 downto i*2),         -- Input write data port, width defined by WRITE_WIDTH parameter
      RDADDR => EventBuf_ra, -- Input read address, width defined by read port depth
      RDCLK => sysclk,   -- 1-bit input read clock
      RDEN => EventBufferRdEn,     -- 1-bit input read port enable
      REGCE => EventBufferREGCE,   -- 1-bit input read output register enable
      RST => '0',       -- 1-bit input reset 
      WE => "1",         -- Input write enable, width defined by write port depth
      WRADDR => EventBuf_wa, -- Input write address, width defined by write port depth
      WRCLK => UsrClk,   -- 1-bit input write clock
      WREN => EventBuf_we(0)      -- 1-bit input write port enable
   );
end generate;
g_AMC_DATA: for i in 0 to 3 generate
	g_AMC_DATA2: for j in 0 to 7 generate
		AMC_DATA(i*16+j*2+1 downto i*16+j*2) <= EventBuf_Do(i*2+j*8+1 downto i*2+j*8);
	end generate;
end generate;
EventBufferRdEn <= AMC_DATA_RdEn or not EventBufDoVld;
EventBufferREGCE <= AMC_DATA_RdEn or not AMC_DATA_vld;
process(sysclk)
begin
	if(sysclk'event and sysclk = '1')then
		if(reset = '1')then
			EventBuf_wa1_SyncRegs <= (others => '0');
			EventBuf_wap <= (others => '0');
		else
			EventBuf_wa1_SyncRegs <= EventBuf_wa1_SyncRegs(2 downto 0) & EventBuf_wa(1);
			if(EventBuf_wa1_SyncRegs(3 downto 2) = "10")then
				EventBuf_wap <= EventBuf_wap + 1;
			end if;
		end if;
		if(reset = '1')then
			EventBufDoVld <= '0';
		elsif(EventBuf_wap /= EventBuf_ra)then
			EventBufDoVld <= '1';
		elsif(AMC_DATA_vld = '0' or AMC_DATA_RdEn = '1')then
			EventBufDoVld <= '0';
		end if;
		if(reset = '1')then
			AMC_DATA_vld <= '0';
		elsif(EventBufDoVld = '1')then
			AMC_DATA_vld <= '1';
		elsif(AMC_DATA_RdEn = '1')then
			AMC_DATA_vld <= '0';
		end if;
	end if;
end process;
process(UsrClk)
begin
	if(UsrClk'event and UsrClk = '1')then
		if(((test = '0' and check_packet = '1' and eof = '1') or (test = '1' and fake_accept = '1' and AMC_en = '1')) and EventBuf_wa(1 downto 0) /= "00")then
			padding <= '1';
			pad_cntr <= EventBuf_wa(1 downto 0);
		else
			if(abort = '1' or pad_cntr = "11")then
				padding <= '0';
			end if;
			pad_cntr(1) <= not abort and padding and (pad_cntr(1) xor pad_cntr(0));
			pad_cntr(0) <= not abort and padding and not pad_cntr(0);
		end if;
		if(test = '1')then
			we_EventInfo <= fake_accept and AMC_en;
		else
			we_EventInfo <= accept and got_eof;
		end if;
		if(test = '1')then
			EventInfoDi(12) <= '0';
		elsif(EventBuf_we = "1")then
			EventInfoDi(12) <= or_reduce(AMCCRC) and AMC_en;
		end if;
		if(InitLink = '1' and ACK = '1')then
			if(CTRversion = RxType(7 downto 0))then
				bad_AMC <= '0';
			else
				bad_AMC <= '1';
			end if;
		end if;
		EventInfoDi(13) <= not(RxType(2) and RxType(1) and RxType(0)) and not test and AMC_en;
		EventInfoDi(14) <= AMC_en;
		if(InitLink = '1')then
			BOE <= (others => '0');
		elsif(NewEvent = '1')then
			BOE <= EventBuf_wa(11 downto 0);
		end if;
		EventInfoDi(11 downto 0) <= EventBuf_wa(11 downto 0) - BOE;
		if(padding = '1')then
			EventBuf_Di <= (others => '0');
		elsif(test = '1')then
			if(fake_header = '1')then
				EventBuf_Di <= x"0" & AMC_ID & fake_DATA(7 downto 0);
				fake_evn <= fake_DATA(3 downto 0);
			else
				EventBuf_Di <= fake_DATA;
			end if;
		else
			EventBuf_Di <= rfifo;
		end if;
		if(test = '1')then
			EventBuf_we(0) <= (fake_WrEn or padding) and AMC_en;
		elsif((padding = '1' or (we_rfifo = '1' and dl_cntr(2) = '1' and TypeData = '1')) and abort = '0' and EventBuf_full = '0')then
			EventBuf_we(0) <= '1';
		else
			EventBuf_we(0) <= '0';
		end if;
		if(dl_cntr(1 downto 0) = "11" and we_rfifo = '1' and TypeData = '1')then
			save_start_addr <= '1';
		else
			save_start_addr <= '0';
		end if;
		if(InitLink = '1')then
			EventInfo_a <= (others => '1');
		elsif(we_EventInfo = '1' and re_EventInfo = '0')then
			EventInfo_a <= EventInfo_a + 1;
		elsif(we_EventInfo = '0' and re_EventInfo = '1')then
			EventInfo_a <= EventInfo_a - 1;
		end if;
		if(EventInfo_a = x"e")then
			EventInfo_full <= '1';
		else
			EventInfo_full <= '0';
		end if;
		re_EventInfo <= EventInfoRdDoneToggleSyncRegs(3) xor EventInfoRdDoneToggleSyncRegs(2);
		if(InitLink = '1')then
			EventInfoRdDoneToggleSyncRegs <= (others => '0');
		else
			EventInfoRdDoneToggleSyncRegs <= EventInfoRdDoneToggleSyncRegs(2 downto 0) & EventInfoRdDoneToggle;
		end if;
		if(resetCntrCycle = '1')then
			CntrBuf_Di <= (others => '0');
			we_CntrBuf <= '1';
		else
			CntrBuf_Di <= rfifo;
			we_CntrBuf <= we_rfifo and dl_cntr(2) and TypeCntr and not abort;
		end if;
--		if(InitLink = '1' or abort = '1')then
		if(Receiving = '0')then
			CntrBuf_wa(4 downto 0) <= (others => '0');
		elsif(we_CntrBuf = '1')then
			CntrBuf_wa(4 downto 0) <= CntrBuf_wa(4 downto 0) + 1;
		end if;
		if(InitLink = '1')then
			CntrBuf_wa(5) <= '0';
		elsif(CntrAccept = '1')then
			CntrBuf_wa(5) <= not CntrBuf_wa(5);
		end if;
		if(InitLink = '1' or resetCntr = '1')then
			CntrBuf_valid <= '0';
		elsif(CntrAccept = '1')then
			CntrBuf_valid <= '1';
		end if;
	end if;
end process;
i_NewEvent : SRL16E
   port map (
      Q => NewEvent,       -- SRL data output
      A0 => '1',     -- Select[0] input
      A1 => '1',     -- Select[1] input
      A2 => '0',     -- Select[2] input
      A3 => '0',     -- Select[3] input
      CE => '1',     -- Clock enable input
      CLK => UsrClk,   -- Clock input
      D => we_EventInfo        -- SRL data input
   );
g_EventInfo : for i in 0 to 19 generate
   i_EventInfo : SRL16E
   port map (
      Q => EventInfoDo(i),       -- SRL data output
      A0 => EventInfo_a(0),     -- Select[0] input
      A1 => EventInfo_a(1),     -- Select[1] input
      A2 => EventInfo_a(2),     -- Select[2] input
      A3 => EventInfo_a(3),     -- Select[3] input
      CE => we_EventInfo,     -- Clock enable input
      CLK => UsrClk,   -- Clock input
      D => EventInfoDi(i)        -- SRL data input
   );
end generate;
g_CntrBuf : for i in 0 to 15 generate
   i_CntrBuf : RAM64X1D
   port map (
      DPO => CntrBuf_Do(i),     -- Read-only 1-bit data output
      SPO => open,     -- R/W 1-bit data output
      A0 => CntrBuf_wa(0),       -- R/W address[0] input bit
      A1 => CntrBuf_wa(1),       -- R/W address[1] input bit
      A2 => CntrBuf_wa(2),       -- R/W address[2] input bit
      A3 => CntrBuf_wa(3),       -- R/W address[3] input bit
      A4 => CntrBuf_wa(4),       -- R/W address[4] input bit
      A5 => CntrBuf_wa(5),       -- R/W address[5] input bit
      D => CntrBuf_Di(i),         -- Write 1-bit data input
      DPRA0 => Cntr_ra(0), -- Read-only address[0] input bit
      DPRA1 => Cntr_ra(1), -- Read-only address[1] input bit
      DPRA2 => Cntr_ra(2), -- Read-only address[2] input bit
      DPRA3 => Cntr_ra(3), -- Read-only address[3] input bit
      DPRA4 => Cntr_ra(4), -- Read-only address[4] input bit
      DPRA5 => CntrBuf_ra5, -- Read-only address[5] input bit
      WCLK => UsrClk,   -- Write clock input
      WE => we_CntrBuf       -- Write enable input
   );
end generate;
CntrBuf_ra5 <= not CntrBuf_wa(5);
g_CntrBuf2 : for i in 0 to 15 generate
   i_CntrBuf2 : RAM32X1D
   port map (
      DPO => CntrBuf2_Do(i),     -- Read-only 1-bit data output
      SPO => CntrBuf2_SPO(i),     -- R/W 1-bit data output
      A0 => CntrBuf2_wa(0),       -- R/W address[0] input bit
      A1 => CntrBuf2_wa(1),       -- R/W address[1] input bit
      A2 => CntrBuf2_wa(2),       -- R/W address[2] input bit
      A3 => CntrBuf2_wa(3),       -- R/W address[3] input bit
      A4 => CntrBuf2_wa(4),       -- R/W address[4] input bit
      D => CntrBuf2_Di(i),         -- Write 1-bit data input
      DPRA0 => Cntr_ra(0), -- Read-only address[0] input bit
      DPRA1 => Cntr_ra(1), -- Read-only address[1] input bit
      DPRA2 => Cntr_ra(2), -- Read-only address[2] input bit
      DPRA3 => Cntr_ra(3), -- Read-only address[3] input bit
--      DPRA4 => CntrBuf2_ra4, -- Read-only address[4] input bit
      DPRA4 => Cntr_ra(4), -- Read-only address[4] input bit
      WCLK => UsrClk,   -- Write clock input
      WE => we_CntrBuf2       -- Write enable input
   );
end generate;
--CntrBuf2_ra4 <= not CntrBuf2_wa(4);
process(UsrClk)
begin
	if(UsrClk'event and UsrClk = '1')then
		we_CntrBuf2p <= not we_CntrBuf2;
		we_CntrBuf2 <= we_CntrBuf2p and not we_CntrBuf2;
		if(resetCntr = '1')then
			CntrBuf2_wa <= (others => '0');
		elsif(we_CntrBuf2 = '1')then
			CntrBuf2_wa <= CntrBuf2_wa + 1;
		end if;
		if(resetCntr = '1')then
			resetCntrCycle <= '1';
		elsif(we_CntrBuf2 = '1' and and_reduce(CntrBuf2_wa) = '1')then
			resetCntrCycle <= '0';
		end if;
		if(CntrBuf2_wa(4) = '0')then
			case CntrBuf2_wa(3 downto 0) is
				when x"1" => Cntrs <= sglErrCntr;
				when x"2" => Cntrs <= dblErrCntr;
				when x"3" => Cntrs <= BC0mmCntr;
				when x"4" => Cntrs <= BcntMmCntr;
				when x"5" => Cntrs <= ResendCntr;
				when x"6" => Cntrs <= AcceptCntr;
				when x"7" => Cntrs <= CntrAcceptCntr;
				when x"8" => Cntrs <= ACKcntr;
				when x"9" => Cntrs <= RxEventCntr;
				when x"a" => Cntrs <= RdEventCntr;
				when x"b" => Cntrs <= DataAbortCntr;
				when x"c" => Cntrs <= CntrAbortCntr;
				when x"d" => Cntrs <= ACKNUM_fullAbortCntr;
				when x"e" => Cntrs <= EventBuf_fullAbortCntr;
				when others => Cntrs <= EventInfo_fullAbortCntr;
			end case;
		else
			case CntrBuf2_wa(3 downto 0) is
				when x"0" => Cntrs <= SEQAbortCntr;
				when x"1" => Cntrs <= CRCAbortCntr;
				when x"2" => Cntrs <= frameAbortCntr;
				when x"3" => Cntrs <= bad_KAbortCntr;
				when x"4" => Cntrs <= BUSYCntr;
				when x"5" => Cntrs <= EvtEVNmmCntr;
				when x"6" => Cntrs <= EvtBCNmmCntr;
				when x"7" => Cntrs <= EvtOCNmmCntr;
				when x"8" => Cntrs <= EventBuf_wa(6 downto 0);
				when x"9" => Cntrs <= EventBuf_ra(4 downto 0) & "00";
				when x"a" => Cntrs <= EventBuf_rap(2 downto 0) & x"0";
				when x"b" => Cntrs <= L1Ainfo_wa(6 downto 0);
				when x"c" => Cntrs <= L1Ainfo_ra(6 downto 0);
				when x"d" => Cntrs <= L1Ainfo_wap(6 downto 0);
				when x"e" => Cntrs <= RxResetDone & qpll_lock & InitLink & TxState;
				when others => Cntrs <= AMCRdy & '0' & got_AMC_BC0 & TTC_lock & BC0_lock & EventInfoRdDoneToggle & EventInfoToggle;
			end case;
		end if;
		CntrBuf2_SPO_q <= CntrBuf2_SPO;
		if(resetCntrCycle = '1')then
			CntrBuf2_Di(15 downto 7) <= (others => '0');
		elsif(CntrBuf2_wa(4 downto 3) = "11")then
			case CntrBuf2_wa(2 downto 0) is
				when "000" => CntrBuf2_Di(15 downto 7) <= EventBuf_ovf_flag & '0' & EventBuf_wa(13 downto 7);
				when "001" => CntrBuf2_Di(15 downto 7) <= "00" & EventBuf_ra(11 downto 5);
				when "010" => CntrBuf2_Di(15 downto 7) <= "00" & EventBuf_rap(9 downto 3);
				when "011" => CntrBuf2_Di(15 downto 7) <= "000000" & L1Ainfo_wa(9 downto 7);
				when "100" => CntrBuf2_Di(15 downto 7) <= "000000" & L1Ainfo_ra(9 downto 7);
				when "101" => CntrBuf2_Di(15 downto 7) <= AMC_L1A_full & '0' & AMC_L1A & L1Ainfo_wap(7);
				when "110" => CntrBuf2_Di(15 downto 7) <= ReSendQue_a & rfifo_a & ACKNUM_a & reset_SyncRegs(3) & AMCRdy & txfsmresetdone;
				when others => CntrBuf2_Di(15 downto 7) <= (others => '0');
			end case;
		elsif(Cntrs < CntrBuf2_SPO_q(6 downto 0))then
			CntrBuf2_Di(15 downto 7) <= CntrBuf2_SPO_q(15 downto 7) + 1;
		else
			CntrBuf2_Di(15 downto 7) <= CntrBuf2_SPO_q(15 downto 7);
		end if;
		if(resetCntrCycle = '1')then
			CntrBuf2_Di(6 downto 0) <= (others => '0');
		else
			CntrBuf2_Di(6 downto 0) <= Cntrs;
		end if;
	end if;
end process;
ipb_rdata <= (others => '0');
process(sysclk)
begin
	if(sysclk'event and sysclk = '1')then
		if(AMC_en = '0')then
			EventInfo <= (others => '0');
		elsif(EventInfo_dav_i = '0')then
			EventInfo <= EventInfoDo;
		end if;
	end if;
end process;
process(sysclk,InitLink)
begin
	if(InitLink = '1')then
		EventInfoRdDoneToggle <= '0';
		EventBuf_ra <= (others => '0');
		EventInfo_dav_i <= '0';
		EventInfoToggleSyncRegs <= (others => '0');
		L1Ainfo_wa <= (others => '0');
	elsif(sysclk'event and sysclk = '1')then
		if(reset = '1')then
			EventInfoRdDoneToggle <= '0';
			EventBuf_ra <= (others => '0');
			EventInfo_dav_i <= '0';
			EventInfoToggleSyncRegs <= (others => '0');
			L1Ainfo_wa <= (others => '0');
		else
			if(EventInfoRdDone = '1')then
				EventInfoRdDoneToggle <= not EventInfoRdDoneToggle;
			end if;
			if((AMC_DATA_RdEn = '1' or EventBufDoVld = '0') and EventBuf_wap /= EventBuf_ra)then
				EventBuf_ra <= EventBuf_ra + 1;
			end if;
			if(AMC_en = '0' or EventInfoRdDone = '1')then
				EventInfo_dav_i <= '0';
			elsif(EventInfoToggleSyncRegs(3) /= EventInfoToggleSyncRegs(2))then
				EventInfo_dav_i <= '1';
			end if;
			EventInfoToggleSyncRegs <= EventInfoToggleSyncRegs(2 downto 0) & EventInfoToggle;
			if(L1A_WrEn = '1')then
				L1Ainfo_wa <= L1Ainfo_wa + 1;
			end if;
		end if;
	end if;
end process;
i_L1Ainfo : BRAM_SDP_MACRO
   generic map (
      BRAM_SIZE => "18Kb", -- Target BRAM, "18Kb" or "36Kb" 
      DEVICE => "7SERIES", -- Target device: "VIRTEX5", "VIRTEX6", "7SERIES", "SPARTAN6" 
      WRITE_WIDTH => 16,    -- Valid values are 1-72 (37-72 only valid when BRAM_SIZE="36Kb")
      READ_WIDTH => 16,     -- Valid values are 1-72 (37-72 only valid when BRAM_SIZE="36Kb")
      DO_REG => 0, -- Optional output register (0 or 1)
      SIM_COLLISION_CHECK => "NONE", -- Collision check enable "ALL", "WARNING_ONLY", 
                                    -- "GENERATE_X_ONLY" or "NONE"       
      WRITE_MODE => "WRITE_FIRST", -- Specify "READ_FIRST" for same clock or synchronous clocks
                                   --  Specify "WRITE_FIRST for asynchrononous clocks on ports
      INIT => X"000000000000000000") --  Initial values on output port
   port map (
      DO => L1AinfoDo,         -- Output read data port, width defined by READ_WIDTH parameter
      DI => L1A_DATA,         -- Input write data port, width defined by WRITE_WIDTH parameter
      RDADDR => L1Ainfo_ra, -- Input read address, width defined by read port depth
      RDCLK => UsrClk,   -- 1-bit input read clock
      RDEN => '1',     -- 1-bit input read port enable
      REGCE => '1',   -- 1-bit input read output register enable
      RST => '0',       -- 1-bit input reset 
      WE => "11",         -- Input write enable, width defined by write port depth
      WRADDR => L1Ainfo_wa, -- Input write address, width defined by write port depth
      WRCLK => sysclk,   -- 1-bit input write clock
      WREN => L1A_WrEn      -- 1-bit input write port enable
   );
--process(UsrClk)
--begin
--	if(UsrClk'event and UsrClk = '1')then
--		L1AinfoDo <= L1Ainfo(conv_integer(L1Ainfo_ra));
--	end if;
--end process;
--process(sysclk)
--begin
--	if(sysclk'event and sysclk = '1')then
--		if(L1A_WrEn = '1')then
--			L1Ainfo(conv_integer(L1Ainfo_wa)) <= L1A_DATA;
--		end if;
--	end if;
--end process;
g_ReSendQueue : for i in 0 to 15 generate
   i_ReSendQueue : SRL16E
   port map (
      Q => ReSendQueOut(i),       -- SRL data output
      A0 => ReSendQue_a(0),     -- Select[0] input
      A1 => ReSendQue_a(1),     -- Select[1] input
      A2 => '0',     -- Select[2] input
      A3 => '0',     -- Select[3] input
      CE => we_ReSendQue,     -- Clock enable input
      CLK => UsrClk,   -- Clock input
      D => ReSendQueIn(i)        -- SRL data input
   );
end generate;
ReSendQueIn <= SEQNUM & L1Ainfo_start;
-- Tx logic
process(UsrClk)
begin
	if(UsrClk'event and UsrClk = '1')then
		if(InitLink = '1')then
			L1Ainfo_ra <= (others => '0');
		elsif(ReSend = '1')then
			L1Ainfo_ra <= ReSendQueOut(7 downto 0) & "00";
		elsif(ec_L1Ainfo_ra = '1')then
			L1Ainfo_ra <= L1Ainfo_ra + 1;
		end if;
		if(InitLink = '1')then
			L1Ainfo_start <= (others => '0');
		elsif(ReSend = '1')then
			L1Ainfo_start <= ReSendQueOut(7 downto 0);
		elsif(we_ReSendQue = '1')then
			L1Ainfo_start <= L1Ainfo_ra(9 downto 2);
		end if;
		if(InitLink = '1')then
			L1Ainfo_wa2SyncRegs <= (others => '0');
		else
			L1Ainfo_wa2SyncRegs <= L1Ainfo_wa2SyncRegs(2 downto 0) & L1Ainfo_wa(2);
		end if;
		if(L1Ainfo_ra(1 downto 0) = "00" and ec_L1Ainfo_ra = '1')then
			L1AinfoSent <= '1';
		else
			L1AinfoSent <= '0';
		end if;
		if(InitLink = '1')then
			L1Ainfo_wap <= (others => '0');
		elsif(L1Ainfo_wa2SyncRegs(3) /= L1Ainfo_wa2SyncRegs(2))then
			L1Ainfo_wap <= L1Ainfo_wap + 1;
		end if;
		if(InitLink = '1' or L1Ainfo_wap = L1Ainfo_ra(9 downto 2))then
			L1Ainfo_empty <= '1';
		else
			L1Ainfo_empty <= '0';
		end if;
		if(InitLink = '1')then
			AMC_L1A <= (others => '0');
		elsif(L1AinfoSent = '1' and we_EventInfo = '0')then
			AMC_L1A <= AMC_L1A + 1;
		elsif(L1AinfoSent = '0' and we_EventInfo = '1')then
			AMC_L1A <= AMC_L1A - 1;
		end if;
		AMC_L1A_full <= and_reduce(AMC_L1A(5 downto 2));
		if(reset_SyncRegs(3) = '1')then
			InitLink <= '1';
		elsif(ACK = '1' or test = '1')then
			InitLink <= '0';
		end if;
		if(reset_SyncRegs(3) = '1' or ReSend = '1')then
			ReSendQue_a <= (others => '1');
		else
			ReSendQue_a(1) <= ReSendQue_a(1) xor ((ReSendQue_a(0) and not ACK and we_ReSendQue) or (not ReSendQue_a(0) and ACK and not we_ReSendQue));
			ReSendQue_a(0) <= ReSendQue_a(0) xor we_ReSendQue xor ACK;
		end if;
		if(TxState = SendWC and Resend = '0'and (L1ASent = '1' or InitLink = '1'))then
			we_ReSendQue <= '1';
		else
			we_ReSendQue <= '0';
		end if;
		if(ReSendQue_a = "10")then
			ReSendQue_full <= '1';
		else
			ReSendQue_full <= '0';
		end if;
		if(reset_SyncRegs(3) = '1' or ReSend = '1')then
			ReSendQue_empty <= '1';
		elsif(we_ReSendQue = '1')then
			ReSendQue_empty <= '0';
		elsif(ACK = '1' and ReSendQue_a = "00")then
			ReSendQue_empty <= '1';
		end if;
		if(InitLink = '1')then
			SEQNUM <= x"01";
		elsif(ReSend = '1')then
			SEQNUM <= ReSendQueOut(15 downto 8);
		elsif(we_ReSendQue = '1')then
			SEQNUM <= SEQNUM(6 downto 0) & not(SEQNUM(7) xor SEQNUM(5) xor SEQNUM(4) xor SEQNUM(3));
		end if;
		if(reset_SyncRegs(3) = '1')then
			got_comma <= '0';
		elsif(RXCHARISCOMMA = "11")then
			got_comma <= '1';
		end if;
		if(AMCRdy = '0' or reset_SyncRegs(3) = '1' or ReSend = '1')then
			TxState <= IDLE;
			ec_L1Ainfo_ra <= '0';
		else
			case TxState is
				when IDLE => -- send R_word (idle)
					TxState <= SendK;
					ec_L1Ainfo_ra <= '0';
				when SendK => -- send K_word
					if((ReSend_l = '1' and InitLink = '1') or (ReSendQue_full = '0' and L1Ainfo_empty = '0' and AMC_L1A_full = '0') or ACKNUM_empty = '0' or GotCntr = '1')then
						TxState <= SendSEQ;
					else
						TxState <= IDLE;
					end if;
					if(InitLink = '1')then
						L1ASent <= '0';
						IsACK <= '0';
						TxType <= InitRqst;
					elsif(GotCntr = '1')then
						L1ASent <= '0';
						IsACK <= '0';
						TxType <= Counter;
					elsif(ReSendQue_full = '0' and L1Ainfo_empty = '0' and AMC_L1A_full = '0')then
						L1ASent <= '1';
						IsACK <= '0';
						TxType <= data;
					else
						L1ASent <= '0';
						IsACK <= '1';
						TxType <= Acknowledge;
					end if;
				when SendSEQ => -- send packet type
					TxState <= SendType;
					if(IsACK = '0')then
						ACKNUM_MUX <= CntrACKNUM;
					else
						ACKNUM_MUX <= ACKNUM_l;
					end if;
					ec_L1Ainfo_ra <= L1Asent;
				when SendType => -- send packet_seq
					if(L1Asent = '0')then
						TxState <= SendWC;
					else
						TxState <= SendData;
					end if;
				when SendWC => -- send packet_wc
					TxState <= WaitCRC;
				when WaitCRC => -- wait for CRC
					TxState <= SendCRC;
				when SendData => -- send L1A info
					if(packet_wc = "11")then
						TxState <= SendWC;
					end if;
					if(packet_wc = "10")then
						ec_L1Ainfo_ra <= '0';
					end if;
				when others => TxState <= IDLE;
			end case;
		end if;
		if(TxState = SendK)then 
			packet_wc <= (others => '0');
		elsif(TxState = SendData)then
			packet_wc(1) <= packet_wc(1) xor packet_wc(0);
			packet_wc(0) <= not packet_wc(0);
		end if;
		case TxState is
			when SendK => TXDATA_i <= K_word;
			when SendData => TXDATA_i <= L1AinfoDo;
			when SendSEQ => TXDATA_i <= SEQNUM & TxType;
			when SendType => TXDATA_i <= ACKNUM_MUX & x"0" & AMC_ID;
			when SendWC => TXDATA_i <= x"000" & '0' & L1Asent & "00";
			when SendCRC => TXDATA_i <= TxCRC;
			when others => TXDATA_i <= R_word;
		end case;
		case TxState is
			when IDLE | SendK | WaitCRC => TxIsK <= '1';
			when others => TxIsK <= '0';
		end case;
		if(TxState = IDLE or TxState = SendK)then
			Init_TxCRC <= '1';
		else
			Init_TxCRC <= '0';
		end if;
	end if;
end process;
i_TxCRC: crc16D16 PORT MAP(
		clk => UsrClk,
		init_crc => Init_TxCRC,
		we_crc => we_TxCRC,
		d => TXDATA_i,
		crc => TxCRC
	);
we_TxCRC <= not TxIsK;
TXCHARISK <= "11" when TxIsK = '1' else "00";
end Behavioral;

