----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    09:38:09 08/14/2013 
-- Design Name: 
-- Module Name:    FIFO65x1536 - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
Library UNIMACRO;
use UNIMACRO.vcomponents.all;

entity FIFO66x1536 is
    Port ( c : in  STD_LOGIC;
           rst : in  STD_LOGIC;
           en : in  STD_LOGIC;
           di : in  STD_LOGIC_VECTOR (65 downto 0);
           WREN : in  STD_LOGIC;
           RDEN : in  STD_LOGIC;
           empty : out  STD_LOGIC;
           full : out  STD_LOGIC;
           do : out  STD_LOGIC_VECTOR (65 downto 0));
end FIFO66x1536;

architecture Behavioral of FIFO66x1536 is
signal RDCOUNT1 : std_logic_vector(8 downto 0) := (others => '0');
signal WRCOUNT1 : std_logic_vector(8 downto 0) := (others => '0');
signal RDCOUNT2 : std_logic_vector(9 downto 0) := (others => '0');
signal WRCOUNT2 : std_logic_vector(9 downto 0) := (others => '0');
signal RDCOUNT3 : std_logic_vector(9 downto 0) := (others => '0');
signal WRCOUNT3 : std_logic_vector(9 downto 0) := (others => '0');
signal FIFO_DO : std_logic_vector(65 downto 0) := (others => '0');
signal FIFO_WREN : std_logic := '0';
signal FIFO_full : std_logic := '0';
signal FIFO_empty : std_logic := '0';

begin
i_FIFO_out : FIFO_DUALCLOCK_MACRO
   generic map (
      DEVICE => "7SERIES",            -- Target Device: "VIRTEX5", "VIRTEX6", "7SERIES" 
      ALMOST_FULL_OFFSET => X"0008",  -- Sets almost full threshold
      ALMOST_EMPTY_OFFSET => X"0080", -- Sets the almost empty threshold
      DATA_WIDTH => 66,   -- Valid values are 1-72 (37-72 only valid when FIFO_SIZE="36Kb")
      FIFO_SIZE => "36Kb",            -- Target BRAM, "18Kb" or "36Kb" 
      FIRST_WORD_FALL_THROUGH => TRUE) -- Sets the FIFO FWFT to TRUE or FALSE
   port map (
      ALMOSTEMPTY => open,   -- 1-bit output almost empty
      ALMOSTFULL => open,     -- 1-bit output almost full
      DO => do,                     -- Output data, width defined by DATA_WIDTH parameter
      EMPTY => empty,               -- 1-bit output empty
      FULL => FIFO_full,                 -- 1-bit output full
      RDCOUNT => RDCOUNT1,           -- Output read count, width determined by FIFO depth
      RDERR => open,               -- 1-bit output read error
      WRCOUNT => WRCOUNT1,           -- Output write count, width determined by FIFO depth
      WRERR => open,               -- 1-bit output write error
      DI => FIFO_DO,                     -- Input data, width defined by DATA_WIDTH parameter
      RDCLK => c,               -- 1-bit input read clock
      RDEN => RDEN,                 -- 1-bit input read enable
      RST => rst,                   -- 1-bit input reset
      WRCLK => c,               -- 1-bit input write clock
      WREN => FIFO_WREN                  -- 1-bit input write enable
   );
FIFO_WREN <= not FIFO_empty and not FIFO_full and en;
i_FIFO_ih : FIFO_DUALCLOCK_MACRO
   generic map (
      DEVICE => "7SERIES",            -- Target Device: "VIRTEX5", "VIRTEX6", "7SERIES" 
      ALMOST_FULL_OFFSET => X"03f0",  -- Sets almost full threshold
      ALMOST_EMPTY_OFFSET => X"0080", -- Sets the almost empty threshold
      DATA_WIDTH => 33,   -- Valid values are 1-72 (37-72 only valid when FIFO_SIZE="36Kb")
      FIFO_SIZE => "36Kb",            -- Target BRAM, "18Kb" or "36Kb" 
      FIRST_WORD_FALL_THROUGH => TRUE) -- Sets the FIFO FWFT to TRUE or FALSE
   port map (
      ALMOSTEMPTY => open,   -- 1-bit output almost empty
      ALMOSTFULL => full,     -- 1-bit output almost full
      DO => FIFO_DO(65 downto 33),                     -- Output data, width defined by DATA_WIDTH parameter
      EMPTY => FIFO_empty,               -- 1-bit output empty
      FULL => open,                 -- 1-bit output full
      RDCOUNT => RDCOUNT2,           -- Output read count, width determined by FIFO depth
      RDERR => open,               -- 1-bit output read error
      WRCOUNT => WRCOUNT2,           -- Output write count, width determined by FIFO depth
      WRERR => open,               -- 1-bit output write error
      DI => di(65 downto 33),                     -- Input data, width defined by DATA_WIDTH parameter
      RDCLK => c,               -- 1-bit input read clock
      RDEN => FIFO_WREN,                 -- 1-bit input read enable
      RST => rst,                   -- 1-bit input reset
      WRCLK => c,               -- 1-bit input write clock
      WREN => WREN                  -- 1-bit input write enable
   );
i_FIFO_il : FIFO_DUALCLOCK_MACRO
   generic map (
      DEVICE => "7SERIES",            -- Target Device: "VIRTEX5", "VIRTEX6", "7SERIES" 
      ALMOST_FULL_OFFSET => X"0008",  -- Sets almost full threshold
      ALMOST_EMPTY_OFFSET => X"0080", -- Sets the almost empty threshold
      DATA_WIDTH => 33,   -- Valid values are 1-72 (37-72 only valid when FIFO_SIZE="36Kb")
      FIFO_SIZE => "36Kb",            -- Target BRAM, "18Kb" or "36Kb" 
      FIRST_WORD_FALL_THROUGH => TRUE) -- Sets the FIFO FWFT to TRUE or FALSE
   port map (
      ALMOSTEMPTY => open,   -- 1-bit output almost empty
      ALMOSTFULL => open,     -- 1-bit output almost full
      DO => FIFO_DO(32 downto 0),                     -- Output data, width defined by DATA_WIDTH parameter
      EMPTY => open,               -- 1-bit output empty
      FULL => open,                 -- 1-bit output full
      RDCOUNT => RDCOUNT3,           -- Output read count, width determined by FIFO depth
      RDERR => open,               -- 1-bit output read error
      WRCOUNT => WRCOUNT3,           -- Output write count, width determined by FIFO depth
      WRERR => open,               -- 1-bit output write error
      DI => di(32 downto 0),                     -- Input data, width defined by DATA_WIDTH parameter
      RDCLK => c,               -- 1-bit input read clock
      RDEN => FIFO_WREN,                 -- 1-bit input read enable
      RST => rst,                   -- 1-bit input reset
      WRCLK => c,               -- 1-bit input write clock
      WREN => WREN                  -- 1-bit input write enable
   );


end Behavioral;

