----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    11:02:29 01/25/2013 
-- Design Name: 
-- Module Name:    ddr_wportA - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
use IEEE.std_logic_misc.all;
use work.amc13_pack.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;
Library UNIMACRO;
use UNIMACRO.vcomponents.all;

entity ddr_rport is
    Port ( memclk : in  STD_LOGIC;
           sysclk : in  STD_LOGIC;
           reset : in  STD_LOGIC;
           resetSys : in  STD_LOGIC;
           resetMem : in  STD_LOGIC;
           run : in  STD_LOGIC;
           test : in  STD_LOGIC_VECTOR(1 downto 0); -- memory test
					 test_block_sent	: in  STD_LOGIC; -- indicating test writing progress. One pulse every 256 32-bit words
					 test_pause : OUT std_logic;
					 test_status	: out  STD_LOGIC_VECTOR(63 downto 0);
-- each TCP segment sent will have dest IP address (32-bit), TCP port(2x16-bit), SEQ number(32-bit), length(16 bit), time stamp(16 bit), control bits(6)
-- and memory starting address of the segment payload data(24-bit)
-- for alignment purposes, each entry occupies 32 bytes of memory space (256 bits)
           TCP_dout : out  STD_LOGIC_VECTOR(31 downto 0); -- TCP data are written in unit of 32-bit words
           TCP_addr : in  STD_LOGIC_VECTOR(28 downto 0); -- 28-26 encoded request source 25-0 address in 64 bit word
           TCP_length : in  STD_LOGIC_VECTOR(9 downto 0); -- in 64 bit word, actual length - 1
           TCP_dout_valid : out  STD_LOGIC;
           TCP_rqst : in  STD_LOGIC;
           TCP_ack : out  STD_LOGIC;
					 TCP_lastword : out  STD_LOGIC;
--	ipbus signals
					 ipb_clk : in  STD_LOGIC;
					 ipb_write : in  STD_LOGIC;
					 ipb_strobe	: in  STD_LOGIC;
					 ipb_addr	: in  STD_LOGIC_VECTOR(31 downto 0);
					 ipb_rdata : out  STD_LOGIC_VECTOR(31 downto 0);
					 ipb_ack : out  STD_LOGIC;
					 page_addr	: in  STD_LOGIC_VECTOR(11 downto 0); -- monitored event memory window
-- monitoring the write progress
           w_rqst : in  STD_LOGIC_VECTOR (3 downto 0);
           w_ack : in  STD_LOGIC_VECTOR (3 downto 0);
           w_addr : in  STD_LOGIC_VECTOR (23 downto 0); -- in unit of 256-bit word
           w_app_en : in  STD_LOGIC;
           app_rdy : in  STD_LOGIC;
-- ddr3 user interface
           app_rqst : out  STD_LOGIC; -- request to output data
           app_ack : in  STD_LOGIC; -- permission to output data
           app_en_accept : in  STD_LOGIC; -- permission to output data
           app_en : out  STD_LOGIC;
           app_rd_data_valid : in  STD_LOGIC;
					 app_rd_data : in  STD_LOGIC_VECTOR(255 downto 0);
           app_addr : out  STD_LOGIC_VECTOR (27 downto 0) := (others => '0')); -- in unit of 32bit word, bit 27 is not used
end ddr_rport;

architecture Behavioral of ddr_rport is
COMPONENT RAM32x6D
	PORT(
		wclk : IN std_logic;
		rclk : IN std_logic;
		di : IN std_logic_vector(5 downto 0);
		we : IN std_logic;
		wa : IN std_logic_vector(4 downto 0);
		ra : IN std_logic_vector(4 downto 0);
		ceReg : IN std_logic;          
		do : OUT std_logic_vector(5 downto 0)
		);
END COMPONENT;
signal TCPqueue_we : std_logic := '0';
signal ipb_rbuf_wa : std_logic_vector(4 downto 0) := (others => '0');
signal ipb_rbuf_wap : std_logic_vector(1 downto 0) := (others => '0');
signal ipb_rbuf_ra : std_logic_vector(4 downto 0) := (others => '0');
signal ipb_rbuf_do : std_logic_vector(31 downto 0) := (others => '0');
signal ipb_start_addr : std_logic_vector(26 downto 0) := (others => '0');
signal ipb_rbuf_wa4SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal ipb_rbuf_wa3SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal ipbReadSyncRegs : std_logic_vector(3 downto 0) := (others => '0');
signal ipb_rbuf_we : std_logic := '0';
signal ipbRead : std_logic := '0';
signal ipb_ack_i : std_logic := '0';
signal ipb_rqst : std_logic := '0';
signal inc_ipb_rqst_cnt : std_logic := '0';
signal ipb_raddr : std_logic_vector(23 downto 0) := (others => '0');
signal ipb_seq : std_logic_vector(1 downto 0) := (others => '0');
signal ipb_rqst_cnt : std_logic_vector(1 downto 0) := (others => '0');
signal ipb_rqst_inqueue : std_logic_vector(1 downto 0) := (others => '0');
signal ipb_rqstSyncRegs : std_logic_vector(3 downto 0) := (others => '0');
signal ipb_rqst_l : std_logic := '0';
signal queue_we : std_logic := '0';
signal queue_do_vld : std_logic := '0';
signal TCP_addr1and0 : std_logic_vector(1 downto 0) := (others => '0');
signal queue_di : std_logic_vector(28 downto 0) := (others => '0');
signal queue_dop : std_logic_vector(28 downto 0) := (others => '0');
signal queue_do : std_logic_vector(28 downto 0) := (others => '0');
signal queue_wa : std_logic_vector(3 downto 0) := (others => '0');
signal queue_ra : std_logic_vector(3 downto 0) := (others => '0');
signal queue_wap : std_logic_vector(3 downto 0) := (others => '0');
signal queue_wa0SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal queue_wa1SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal queue_rap : std_logic_vector(3 downto 0) := (others => '0');
signal queue_ra0SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal queue_ra1SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal Oqueue_di : std_logic_vector(10 downto 0) := (others => '0');
signal Oqueue_dop : std_logic_vector(10 downto 0) := (others => '0');
signal Oqueue_do : std_logic_vector(10 downto 0) := (others => '0');
signal Oqueue_a : std_logic_vector(3 downto 0) := (others => '1');
signal queue_full : std_logic := '0';
signal Oqueue_re : std_logic := '0';
signal Oqueue_do_vld : std_logic := '0';
signal test_wc : std_logic_vector(5 downto 0) := (others => '0');
signal testSyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal lfsr: std_logic_vector(31 downto 0) := (others => '0');
signal testErr : std_logic := '0';
signal testCntr: std_logic_vector(30 downto 0) := (others => '0');
signal app_rqst_i : std_logic := '0';
signal s_demux : std_logic_vector(2 downto 0) := (others => '0');
signal data_demux : std_logic_vector(31 downto 0) := (others => '0');
signal lengthCntr : std_logic_vector(7 downto 0) := (others => '0');
signal rd_cnt : std_logic_vector(4 downto 0) := (others => '0');
signal app_addr_i : std_logic_vector(27 downto 0) := (others => '0');
signal ec_buf_do : std_logic := '0';
signal buf_do_vld : std_logic := '0';
signal buf_do_vld_q : std_logic := '0';
signal ec_buf_ra : std_logic := '0';
signal buf_wa : std_logic_vector(8 downto 0) := (others => '0');
signal buf_wap : std_logic_vector(8 downto 0) := (others => '0');
signal buf_wa0SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal buf_wa1SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal buf_ra : std_logic_vector(8 downto 0) := (others => '0');
signal buf_do : std_logic_vector(255 downto 0) := (others => '0');
signal TCP_rrqst : std_logic := '0';
signal TCP_raddr : std_logic_vector(23 downto 0) := (others => '0');
signal TCP_raddr_end : std_logic_vector(10 downto 0) := (others => '0');
signal TCP_rlengthCntr : std_logic_vector(10 downto 0) := (others => '0');
signal TCP_rlengthp : std_logic_vector(6 downto 0) := (others => '0');
signal TCP_rlength : std_logic_vector(7 downto 0) := (others => '0');
signal TCPqueue_di : std_logic_vector(36 downto 0) := (others => '0');
signal TCPqueue_do : std_logic_vector(36 downto 0) := (others => '0');
signal TCPqueue_a : std_logic_vector(3 downto 0) := (others => '1');
signal TCPinQueueCnt : std_logic_vector(2 downto 0) := (others => '0');
signal waddrFIFO_di : std_logic_vector(29 downto 0) := (others => '0');
signal waddrFIFO_do : std_logic_vector(29 downto 0) := (others => '0');
signal waddrFIFO_wa : std_logic_vector(4 downto 0) := (others => '0');
signal waddrFIFO_ra : std_logic_vector(4 downto 0) := (others => '0');
signal waddrFIFO_we : std_logic := '0';
signal waddrFIFO_wa2SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal waddrFIFO_wa1SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal waddrFIFO_wa0SyncRegs : std_logic_vector(2 downto 0) := (others => '0');
signal LastWaddr_do : std_logic_vector(23 downto 0) := (others => '0');
signal LastWaddr_wa : std_logic_vector(4 downto 0) := (others => '0');
signal LastWaddr_ra : std_logic_vector(4 downto 0) := (others => '0');
signal LastWaddr_we : std_logic := '0';
signal EventNotReady : std_logic := '0';
signal Last_TCP_addr : std_logic_vector(12 downto 0) := (others => '0');
signal addr_diff : std_logic_vector(21 downto 0) := (others => '0');
--signal TCP_rqst_dl : std_logic_vector(1 downto 0) := (others => '0');
signal en_test_read : std_logic := '0';
COMPONENT chipscope
	generic (N : integer := 5);
	PORT(
		clka : IN std_logic;
		clkb : IN std_logic;
		ina : IN std_logic_vector(135 downto 0);
		inb : IN std_logic_vector(135 downto 0)       
		);
END COMPONENT;
signal ina : std_logic_vector(135 downto 0) := (others => '0');
signal inb : std_logic_vector(135 downto 0) := (others => '0');
begin
--ina(128) <= w_rqst(0);
--ina(64) <= app_rdy;
--ina(63) <= w_app_en;
--ina(62) <= waddrFIFO_we;
--ina(61 downto 35) <= waddrFIFO_di(26 downto 0);
--ina(34 downto 32) <= waddrFIFO_wa(2 downto 0);
--ina(31 downto 28) <= w_rqst;
--ina(27 downto 24) <= w_ack;
--ina(23 downto 0) <= w_addr;
ina(128) <= TCP_rqst;
inb(128) <= TCP_rqst;
inb(127) <= queue_we;
inb(126 downto 123) <= TCPqueue_a;
inb(122 downto 96) <= waddrFIFO_do(26 downto 0);
inb(95) <= LastWaddr_we;
inb(94) <= waddrFIFO_wa2SyncRegs(2);
inb(93) <= waddrFIFO_wa1SyncRegs(2);
inb(92) <= waddrFIFO_wa0SyncRegs(2);
inb(91 downto 89) <= waddrFIFO_ra(2 downto 0);
inb(86 downto 65) <= addr_diff;
inb(64) <= TCPqueue_we;
inb(63) <= TCP_rqst;
inb(62 downto 34) <= TCP_addr;
inb(33 downto 24) <= TCP_length;
inb(23 downto 0) <= LastWaddr_do;
--ina(128) <= app_rqst_i;
--ina(86) <= queue_do_vld;
--ina(85) <= app_en_accept;
--ina(84) <= app_rqst_i;
--ina(83) <= app_rd_data_valid;
--ina(74 downto 46) <= queue_do;
--ina(45 downto 42) <= queue_ra;
--ina(41 downto 38) <= queue_wap;
--ina(37 downto 29) <= buf_wa;
--ina(28 downto 24) <= rd_cnt;
--ina(23 downto 0) <= app_addr_i(26 downto 3);
--inb(128) <= (ipb_addr(27) or ipb_addr(14)) and ipb_strobe;
--inb(120) <= Oqueue_re;
--inb(119 downto 109) <= Oqueue_do;
--inb(108) <= ipb_addr(27);
--inb(107) <= ipb_addr(14);
--inb(106 downto 99) <= lengthCntr;
--inb(98 downto 96) <= s_demux;
--inb(95) <= ec_buf_ra;
--inb(94) <= buf_do_vld;
--inb(93) <= ec_buf_do;
--inb(92) <= Oqueue_do_vld;
--inb(91) <= queue_we;
--inb(90) <= ipb_rqst_l;
--inb(89) <= ipb_rqst;
--inb(88) <= ipb_strobe;
--inb(87) <= ipb_ack_i;
--inb(86) <= ipbread;
--inb(85) <= ipb_rbuf_we;
--inb(84 downto 83) <= ipb_rqst_inqueue;
--inb(82 downto 78) <= ipb_rqst_cnt;
--inb(77 downto 76) <= ipb_seq;
--inb(75 downto 52) <= ipb_raddr;
--inb(51 downto 44) <= ipb_wc;
--inb(43 downto 12) <= ipb_rbuf_do;
--inb(11 downto 7) <= ipb_rbuf_ra;
--inb(6 downto 5) <= ipb_rbuf_wap;
--inb(4 downto 0) <= ipb_rbuf_wa;
--i_chipscope: chipscope generic map(N => 5) PORT MAP(
--		clka => sysclk,
--		clkb => sysclk,
--		ina => ina,
--		inb => inb
--	);
test_pause <= test_wc(5);
ipb_ack <= ipb_ack_i;
app_rqst <= app_rqst_i;
app_en <= app_ack and app_rqst_i;
TCP_ack <= TCPqueue_we;
test_status(31 downto 0) <= testErr & testCntr;
test_status(47 downto 32) <= lfsr(15 downto 0);
test_status(57 downto 48) <= (others => '0');
test_status(63 downto 58) <= test_wc;
app_addr <= app_addr_i;
TCP_dout <= data_demux;
ipb_ack_i <= '0' when ipbRead = '0' or ipb_rbuf_wap = ipb_rbuf_ra(4 downto 3) else ipb_strobe;
process(ipb_clk)
begin
	if(ipb_clk'event and ipb_clk = '1')then
		if(reset = '1' or ipb_rqst = '1' or ipbRead = '0' or ipb_rqst_inqueue = "11")then
			ipb_rqst <= '0';
		else
			ipb_rqst <= '1';
		end if;
		if(ipbRead = '0')then
			ipb_rqst_inqueue <= "00";
		elsif(ipb_rqst = '1')then
			ipb_rqst_inqueue <= ipb_rqst_inqueue + 1;
		elsif(ipb_ack_i = '1' and ipb_rbuf_ra(2 downto 0) = "111")then
			ipb_rqst_inqueue <= ipb_rqst_inqueue - 1;
		end if;
		if(reset = '0' and test(0) = '0' and ipb_strobe = '1' and ipb_write = '0' and (ipb_addr(27) = '1' or ipb_addr(14) = '1'))then
			ipbRead <= '1';
		else
			ipbRead <= '0';
		end if;
		if(ipbRead = '0')then
			if(ipb_addr(27) = '1')then
				ipb_start_addr <= ipb_addr(26 downto 0);
			elsif(ipb_addr(14) = '1')then
				ipb_start_addr <= '0' & page_addr & ipb_addr(13 downto 0);
			end if;
		end if;
		if(ipbRead = '0')then
			ipb_rbuf_ra <= "00" & ipb_addr(2 downto 0);
		elsif(ipb_ack_i = '1')then
			ipb_rbuf_ra(2 downto 0) <= ipb_rbuf_ra(2 downto 0) + 1;
			if(ipb_rbuf_ra(2 downto 0) = "111")then
				ipb_rbuf_ra(4) <= ipb_rbuf_ra(3);
				ipb_rbuf_ra(3) <= not ipb_rbuf_ra(4);
			end if;
		end if;
		if(ipbRead = '0')then
			ipb_rbuf_wa4SyncRegs <= "000";
			ipb_rbuf_wa3SyncRegs <= "000";
			ipb_rbuf_wap <= ipb_addr(4) & (ipb_addr(4) xor ipb_addr(3));
		else
			ipb_rbuf_wa4SyncRegs <= ipb_rbuf_wa4SyncRegs(1 downto 0) & ipb_rbuf_wa(4);
			ipb_rbuf_wa3SyncRegs <= ipb_rbuf_wa3SyncRegs(1 downto 0) & ipb_rbuf_wa(3);
			ipb_rbuf_wap(1) <= ipb_rbuf_wa4SyncRegs(2);
			ipb_rbuf_wap(0) <= ipb_rbuf_wa3SyncRegs(2);
		end if;
	end if;
end process;
g_ipb_rbuf : for i in 0 to 31 generate
   i_ipb_rbuf : RAM32X1D
   port map (
      DPO => ipb_rdata(i),     -- Read-only 1-bit data output
      SPO => open,     -- R/W 1-bit data output
      A0 => ipb_rbuf_wa(0),       -- R/W address[0] input bit
      A1 => ipb_rbuf_wa(1),       -- R/W address[1] input bit
      A2 => ipb_rbuf_wa(2),       -- R/W address[2] input bit
      A3 => ipb_rbuf_wa(3),       -- R/W address[3] input bit
      A4 => ipb_rbuf_wa(4),       -- R/W address[4] input bit
      D => data_demux(i),         -- Write 1-bit data input
      DPRA0 => ipb_rbuf_ra(0), -- Read-only address[0] input bit
      DPRA1 => ipb_rbuf_ra(1), -- Read-only address[1] input bit
      DPRA2 => ipb_rbuf_ra(2), -- Read-only address[2] input bit
      DPRA3 => ipb_rbuf_ra(3), -- Read-only address[3] input bit
      DPRA4 => ipb_rbuf_ra(4), -- Read-only address[4] input bit
      WCLK => sysclk,   -- Write clock input
      WE => ipb_rbuf_we       -- Write enable input
   );
end generate;
process(sysclk)
begin
	if(sysclk'event and sysclk = '1')then
		testSyncRegs <= testSyncRegs(1 downto 0) & test(0);
		ipbReadSyncRegs <= ipbReadSyncRegs(2 downto 0) & ipbRead;
		ipb_rqstSyncRegs <= ipb_rqstSyncRegs(2 downto 0) & ipb_rqst;
		if(ipb_rqstSyncRegs(3 downto 2) = "10")then
			ipb_rqst_l <= '1';
		elsif(queue_we = '0')then
			ipb_rqst_l <= '0';
		end if;
		if(ipbReadSyncRegs(3 downto 2) = "01")then
			ipb_rbuf_wa <= "00000";
		elsif(ipb_rbuf_we = '1')then
			ipb_rbuf_wa(2 downto 0) <= ipb_rbuf_wa(2 downto 0) + 1;
			if(ipb_rbuf_wa(2 downto 0) = "111")then
				ipb_rbuf_wa(4) <= ipb_rbuf_wa(3);
				ipb_rbuf_wa(3) <= not ipb_rbuf_wa(4);
			end if;
		end if;
		if(resetSys = '1' or testSyncRegs(2) = '1')then
			ipb_rbuf_we <= '0';
		elsif(buf_do_vld = '1' and Oqueue_do(0) = '1' and Oqueue_do(2 downto 1) = ipb_seq)then
			ipb_rbuf_we <= '1';
		else
			ipb_rbuf_we <= '0';
		end if;
	end if;
end process;
g_waddrFIFO : for i in 0 to 4 generate
	i_waddrFIFO: RAM32x6D PORT MAP(
		wclk => memclk,
		rclk => sysclk,
		di => waddrFIFO_di(i*6+5 downto i*6),
		we => waddrFIFO_we,
		wa => waddrFIFO_wa,
		ra => waddrFIFO_ra,
		ceReg => '1',
		do => waddrFIFO_do(i*6+5 downto i*6)
	);
end generate;
process(memclk)
begin
	if(memclk'event and memclk = '1')then
		if(w_app_en = '1' and app_rdy = '1')then
			waddrFIFO_di(26) <= not w_ack(3);
			if(w_ack(3) = '0')then
				waddrFIFO_di(25 downto 24) <= w_ack(2 downto 1);
			else
				waddrFIFO_di(25 downto 24) <= w_addr(23 downto 22);
			end if;
			waddrFIFO_di(23 downto 0) <= w_addr;
		end if;
		if(or_reduce(w_ack) = '1' and or_reduce(w_ack and w_rqst) = '0')then
			waddrFIFO_we <= '1';
		else
			waddrFIFO_we <= '0';
		end if;
		if(waddrFIFO_we = '1')then
			case waddrFIFO_wa(2 downto 0) is
				when "000" => waddrFIFO_wa(2 downto 0) <= "001";
				when "001" => waddrFIFO_wa(2 downto 0) <= "011";
				when "011" => waddrFIFO_wa(2 downto 0) <= "010";
				when "010" => waddrFIFO_wa(2 downto 0) <= "110";
				when "110" => waddrFIFO_wa(2 downto 0) <= "111";
				when "111" => waddrFIFO_wa(2 downto 0) <= "101";
				when "101" => waddrFIFO_wa(2 downto 0) <= "100";
				when others => waddrFIFO_wa(2 downto 0) <= "000";
			end case;
		end if;
	end if;
end process;
process(sysclk)
begin
	if(sysclk'event and sysclk = '1')then
		waddrFIFO_wa2SyncRegs <= waddrFIFO_wa2SyncRegs(1 downto 0) & waddrFIFO_wa(2);
		waddrFIFO_wa1SyncRegs <= waddrFIFO_wa1SyncRegs(1 downto 0) & waddrFIFO_wa(1);
		waddrFIFO_wa0SyncRegs <= waddrFIFO_wa0SyncRegs(1 downto 0) & waddrFIFO_wa(0);
		if(waddrFIFO_wa2SyncRegs(2) /= waddrFIFO_ra(2) or waddrFIFO_wa1SyncRegs(2) /= waddrFIFO_ra(1) or waddrFIFO_wa0SyncRegs(2) /= waddrFIFO_ra(0))then
			LastWaddr_we <= '1';
			case waddrFIFO_ra(2 downto 0) is
				when "000" => waddrFIFO_ra(2 downto 0) <= "001";
				when "001" => waddrFIFO_ra(2 downto 0) <= "011";
				when "011" => waddrFIFO_ra(2 downto 0) <= "010";
				when "010" => waddrFIFO_ra(2 downto 0) <= "110";
				when "110" => waddrFIFO_ra(2 downto 0) <= "111";
				when "111" => waddrFIFO_ra(2 downto 0) <= "101";
				when "101" => waddrFIFO_ra(2 downto 0) <= "100";
				when others => waddrFIFO_ra(2 downto 0) <= "000";
			end case;
		else
			LastWaddr_we <= '0';
		end if;
		Last_TCP_addr <= TCP_addr(12 downto 0) + ("000" & TCP_length);
		addr_diff <= LastWaddr_do(21 downto 0) - TCP_addr(23 downto 2);
		if(TCP_addr(28) = '1' and TCP_addr(24 downto 13) = LastWaddr_do(22 downto 11) and Last_TCP_addr(12 downto 2) > LastWaddr_do(10 downto 0))then
			EventNotReady <= '1';
		else
			EventNotReady <= '0';
		end if;
	end if;
end process;
g_LastWaddr : for i in 0 to 3 generate
	i_LastWaddr: RAM32x6D PORT MAP(
		wclk => sysclk,
		rclk => sysclk,
		di => waddrFIFO_do(i*6+5 downto i*6),
		we => LastWaddr_we,
		wa => LastWaddr_wa,
		ra => LastWaddr_ra,
		ceReg => '1',
		do => LastWaddr_do(i*6+5 downto i*6)
	);
end generate;
LastWaddr_wa <= '0' & waddrFIFO_do(26 downto 23);
LastWaddr_ra <= '0' & TCP_addr(28 downto 25);
g_TCPqueue: for i in 0 to 36 generate
	i_TCPqueue : SRL16E
   port map (
      Q => TCPqueue_do(i),       -- SRL data output
      A0 => TCPqueue_a(0),     -- Select[0] input
      A1 => TCPqueue_a(1),     -- Select[1] input
      A2 => TCPqueue_a(2),     -- Select[2] input
      A3 => TCPqueue_a(3),     -- Select[3] input
      CE => TCPqueue_we,     -- Clock enable input
      CLK => sysclk,   -- Clock input
      D => TCPqueue_di(i)        -- SRL data input
   );
end generate;
g_Oqueue: for i in 0 to 10 generate
	i_Oqueue : SRL16E
   port map (
      Q => Oqueue_dop(i),       -- SRL data output
      A0 => Oqueue_a(0),     -- Select[0] input
      A1 => Oqueue_a(1),     -- Select[1] input
      A2 => Oqueue_a(2),     -- Select[2] input
      A3 => Oqueue_a(3),     -- Select[3] input
      CE => queue_we,     -- Clock enable input
      CLK => sysclk,   -- Clock input
      D => Oqueue_di(i)        -- SRL data input
   );
end generate;
g_queue : for i in 0 to 28 generate
   i_queue : RAM32X1D
   port map (
      DPO => queue_dop(i),     -- Read-only 1-bit data output
      SPO => open,     -- R/W 1-bit data output
      A0 => queue_wa(0),       -- R/W address[0] input bit
      A1 => queue_wa(1),       -- R/W address[1] input bit
      A2 => queue_wa(2),       -- R/W address[2] input bit
      A3 => queue_wa(3),       -- R/W address[3] input bit
      A4 => '0',       -- R/W address[4] input bit
      D => queue_di(i),         -- Write 1-bit data input
      DPRA0 => queue_ra(0), -- Read-only address[0] input bit
      DPRA1 => queue_ra(1), -- Read-only address[1] input bit
      DPRA2 => queue_ra(2), -- Read-only address[2] input bit
      DPRA3 => queue_ra(3), -- Read-only address[3] input bit
      DPRA4 => '0', -- Read-only address[4] input bit
      WCLK => sysclk,   -- Write clock input
      WE => queue_we       -- Write enable input
   );
end generate;
TCP_rlengthp <= TCP_rlength(6 downto 0) + "0000011";
queue_full <= '1' when Oqueue_a = x"b" or Oqueue_a = x"c" or Oqueue_a = x"d" or Oqueue_a = x"e" else '0';
process(sysclk)
begin
	if(sysclk'event and sysclk = '1')then
		if(resetSys = '1' or run = '0' or testSyncRegs(2) = '1' or (TCP_addr(28) = '0' and addr_diff(21) = '1') or EventNotReady = '1' or TCPqueue_we = '1' or TCPqueue_a = x"d" or TCPqueue_a = x"e")then
			TCPqueue_we <= '0';
		elsif(TCP_rqst = '1')then
			TCPqueue_we <= '1';
		end if;
		TCPqueue_di <= Last_TCP_addr(10 downto 0) & TCP_addr(25 downto 0);
		if(resetSys = '1')then
			TCPqueue_a <= (others => '1');
		elsif(TCPqueue_we = '1' and (and_reduce(TCPqueue_a) = '1' or TCP_rrqst = '1'))then
			TCPqueue_a <= TCPqueue_a + 1;
		elsif(TCPqueue_we = '0' and and_reduce(TCPqueue_a) = '0' and TCP_rrqst = '0')then
			TCPqueue_a <= TCPqueue_a - 1;
		end if;
		if(resetSys = '1')then
			TCP_rrqst <= '0';
		elsif(queue_we = '1' and Oqueue_di(10) = '1')then
			TCP_rrqst <= '0';
		elsif(and_reduce(TCPqueue_a) = '0')then
			TCP_rrqst <= '1';
		end if;
		if(TCP_rrqst = '0')then
			TCP_addr1and0 <= TCPqueue_do(1 downto 0);
			TCP_raddr <= TCPqueue_do(25 downto 2); -- in unit of 256 bit word
			TCP_raddr_end(10 downto 7) <= TCPqueue_do(36 downto 33) - 1;
			TCP_raddr_end(6 downto 0) <= TCPqueue_do(32 downto 26);
			if(TCPqueue_do(10 downto 7) = TCPqueue_do(36 downto 33))then
				TCP_rlength(7) <= '1';
				TCP_rlength(6 downto 0) <= TCPqueue_do(32 downto 26) - TCPqueue_do(6 downto 0) + 1;
			else
				TCP_rlength(7) <= '0';
				TCP_rlength(6 downto 0) <= "0000000" - TCPqueue_do(6 downto 0);
			end if;
		elsif(queue_we = '1' and Oqueue_di(0) = '0')then
			TCP_addr1and0 <= "00";
			TCP_raddr(23 downto 5) <= TCP_raddr(23 downto 5) + 1;
			TCP_raddr(4 downto 0) <= (others => '0');
			if(TCP_raddr(8 downto 5) = TCP_raddr_end(10 downto 7))then
				TCP_rlength(7) <= '1';
				TCP_rlength(6 downto 0) <= (TCP_raddr_end(6 downto 0) + 1);
			else
				TCP_rlength <= (others => '0');
			end if;
		end if;
		if(resetSys = '1')then
			TCPinQueueCnt <= "001";
		elsif(queue_we = '1' and Oqueue_di(0) = '0' and (Oqueue_re = '0' or Oqueue_do(0) = '1'))then
			TCPinQueueCnt <= TCPinQueueCnt + 1;
		elsif((queue_we = '0' or Oqueue_di(0) = '1') and Oqueue_re = '1' and Oqueue_do(0) = '0')then
			TCPinQueueCnt <= TCPinQueueCnt - 1;
		elsif(Oqueue_do_vld = '0' and Oqueue_a = x"f")then
			TCPinQueueCnt <= "001";
		end if;
		if(resetSys = '1' or Oqueue_do(0) = '1' or buf_do_vld = '0' or Oqueue_do_vld = '0' or Oqueue_re = '1' or (lengthCntr = x"01" and s_demux(2 downto 1) < Oqueue_do(2 downto 1)))then
			TCP_dout_valid <= '0';
		else
			TCP_dout_valid <= '1';
		end if;
		if(ipbReadSyncRegs(3 downto 2) = "10")then -- ipb_seq is used to eject stalemate read data
			ipb_seq <= ipb_seq + 1;
		end if;
		if(resetSys = '1')then
			ipb_raddr <= (others => '0');
		elsif(ipbReadSyncRegs(3 downto 2) = "01")then
			ipb_raddr <= ipb_start_addr(26 downto 3);
		elsif(queue_we = '1' and Oqueue_di(0) = '1')then
			ipb_raddr <= ipb_raddr + 1; 
		end if;
		case s_demux is
			when "000" => data_demux <= buf_do(31 downto 0);
			when "001" => data_demux <= buf_do(63 downto 32);
			when "010" => data_demux <= buf_do(95 downto 64);
			when "011" => data_demux <= buf_do(127 downto 96);
			when "100" => data_demux <= buf_do(159 downto 128);
			when "101" => data_demux <= buf_do(191 downto 160);
			when "110" => data_demux <= buf_do(223 downto 192);
			when others => data_demux <= buf_do(255 downto 224);
		end case;
		if(buf_do_vld = '0' or Oqueue_re = '1')then
			s_demux <= "000";
		else
			s_demux <= s_demux + 1;
		end if;
		if(resetSys = '1' or testSyncRegs(2) = '0')then
			en_test_read <= '0';
		elsif(test_block_sent = '1')then
			en_test_read <= '1';
		end if;
		if(resetSys = '1' or testSyncRegs(2) = '0' or en_test_read = '0')then
			test_wc <= (others => '0');
		elsif(test_block_sent = '1' and queue_we = '0')then
			test_wc <= test_wc + 1;
		elsif(test_block_sent = '0' and queue_we = '1')then
			test_wc <= test_wc - 1;
		end if;
		if(resetSys = '1' or queue_we = '1')then
			queue_we <= '0';
		elsif((or_reduce(test_wc(5 downto 1)) = '1' and queue_full = '0') or ipb_rqst_l = '1' or (TCP_rrqst = '1' and queue_full = '0' and TCPinQueueCnt(2) = '0'))then
			queue_we <= '1';
		else
			queue_we <= '0';
		end if;
		if(testSyncRegs(2) = '1' or ipb_rqst_l = '1')then
			queue_di(23 downto 0) <= ipb_raddr;
			queue_di(28 downto 24) <= "00001";
			Oqueue_di(0) <= '1';
			Oqueue_di(10 downto 1) <= "00000000" & ipb_seq;
		else
			queue_di(23 downto 0) <= TCP_raddr;
			queue_di(28 downto 24) <= TCP_rlengthp(6 downto 2); -- length in 256-bit words
			Oqueue_di(0) <= '0';
			Oqueue_di(2 downto 1) <= TCP_addr1and0;
			Oqueue_di(9 downto 3) <= TCP_rlength(6 downto 0); -- length in 64-bit words
			Oqueue_di(10) <= TCP_rlength(7);
		end if;
		if(resetSys = '1')then
			Oqueue_do_vld <= '0';
		elsif(Oqueue_a /= x"f")then
			Oqueue_do_vld <= '1';
		elsif(Oqueue_re = '1')then
			Oqueue_do_vld <= '0';
		end if;
		if(Oqueue_do_vld = '0' or Oqueue_re = '1')then
			Oqueue_do <= Oqueue_dop;
		end if;
		if(resetSys = '1' or Oqueue_re = '1'  or (lengthCntr = x"01" and s_demux(2 downto 1) < Oqueue_do(2 downto 1)))then
			lengthCntr <= x"01";
		elsif(buf_do_vld = '1')then
			lengthCntr <= lengthCntr + 1;
		end if;
		if(resetSys = '1')then
			Oqueue_re <= '0';
			TCP_lastword <= '0';
		elsif(Oqueue_do_vld = '1' and ((lengthCntr = (Oqueue_do(9 downto 3) & '0') and Oqueue_do(0) = '0') or (Oqueue_do(0) = '1' and s_demux = "111")))then
			Oqueue_re <= '1';
			TCP_lastword <= Oqueue_do(10);
		else
			Oqueue_re <= '0';
			TCP_lastword <= '0';
		end if;
		if(resetSys = '1')then
			Oqueue_a <= (others => '1');
		elsif(queue_we = '1' and (Oqueue_do_vld = '1' or Oqueue_a = x"f") and Oqueue_re = '0')then
			Oqueue_a <= Oqueue_a + 1;
		elsif(queue_we = '0' and Oqueue_a /= x"f" and (Oqueue_do_vld = '0' or Oqueue_re = '1'))then
			Oqueue_a <= Oqueue_a - 1;
		end if;
		if(resetSys = '1')then
			queue_wa <= (others => '0');
		elsif(queue_we = '1')then
			queue_wa(1) <= queue_wa(0);
			queue_wa(0) <= not queue_wa(1);
			if(queue_wa(1 downto 0) = "10")then
				queue_wa(3 downto 2) <= queue_wa(3 downto 2) + 1;
			end if;
		end if;
		if(resetSys = '1')then
			testErr <= '0';
			testCntr <= (others => '0');
		elsif(testSyncRegs(2) = '1' and buf_do_vld_q = '1')then
			if(lfsr /= data_demux)then
				testErr <= '1';
			end if;
			if(testErr = '0')then
				testCntr <= testCntr + 1;
			end if;
		end if;
		buf_do_vld_q <= buf_do_vld;
		if(testSyncRegs(2) = '0')then
			lfsr <= (others => '0');
		elsif(buf_do_vld_q = '1')then
			if(test(1) = '0')then
				lfsr <= lfsr(30 downto 0) & not(lfsr(31) xor lfsr(21) xor lfsr(1) xor lfsr(0));
			else
				lfsr <= lfsr + 1;
			end if;
		end if;
		if(resetSys = '1')then
			queue_ra0SyncRegs <= (others => '0');
			queue_ra1SyncRegs <= (others => '0');
			queue_rap <= (others => '0');
		else
			queue_ra0SyncRegs <= queue_ra0SyncRegs(1 downto 0) & queue_ra(0);
			queue_ra1SyncRegs <= queue_ra1SyncRegs(1 downto 0) & queue_ra(1);
			queue_rap(1) <= queue_ra1SyncRegs(2);
			queue_rap(0) <= queue_ra0SyncRegs(2);
			if(queue_rap(1) = '1' and queue_ra1SyncRegs(2) = '0')then
				queue_rap(3 downto 2) <= queue_rap(3 downto 2) + 1;
			end if;
		end if;
		if(resetSys = '1')then
			buf_wa0SyncRegs <= (others => '0');
			buf_wa1SyncRegs <= (others => '0');
			buf_wap <= (others => '0');
		else
			buf_wa0SyncRegs <= buf_wa0SyncRegs(1 downto 0) & buf_wa(0);
			buf_wa1SyncRegs <= buf_wa1SyncRegs(1 downto 0) & buf_wa(1);
			buf_wap(1) <= buf_wa1SyncRegs(2);
			buf_wap(0) <= buf_wa0SyncRegs(2);
			if(buf_wap(1) = '1' and buf_wa1SyncRegs(2) = '0')then
				buf_wap(8 downto 2) <= buf_wap(8 downto 2) + 1;
			end if;
		end if;
		if(resetSys = '1' or ec_buf_ra = '1')then
			ec_buf_ra <= '0';
		elsif((buf_do_vld = '0' or s_demux = "110") and buf_ra /= buf_wap)then
			ec_buf_ra <= '1';
		end if;
		if(resetSys = '1' or (Oqueue_re = '1' and Oqueue_do(10) = '1'))then
			buf_do_vld <= '0';
		elsif(ec_buf_do = '1')then
			buf_do_vld <= '1';
		elsif(s_demux = "111")then
			buf_do_vld <= '0';
		end if;
		if(resetSys = '1')then
			buf_ra <= (others => '0');
		elsif(ec_buf_ra = '1')then
			buf_ra(1) <= buf_ra(0);
			buf_ra(0) <= not buf_ra(1);
			if(buf_ra(1 downto 0) = "10")then
				buf_ra(8 downto 2) <= buf_ra(8 downto 2) + 1;
			end if;
		end if;
	end if;
end process;
process(memclk)
begin
	if(memclk'event and memclk = '1')then
		if((queue_wap /= queue_ra and queue_do_vld = '0') or (rd_cnt = "00001" and app_en_accept = '1'))then
			queue_do <= queue_dop;
		end if;
		if(resetMem = '1')then
			queue_do_vld <= '0';
		elsif(queue_wap /= queue_ra)then
			queue_do_vld <= '1';
		elsif(rd_cnt = "00001" and app_en_accept = '1')then
			queue_do_vld <= '0';
		end if;
		if(resetMem = '1')then
			queue_ra <= (others => '0');
		elsif(queue_wap /= queue_ra and (queue_do_vld = '0' or (rd_cnt = "00001" and app_en_accept = '1')))then
			queue_ra(1) <= queue_ra(0);
			queue_ra(0) <= not queue_ra(1);
			if(queue_ra(1 downto 0) = "10")then
				queue_ra(3 downto 2) <= queue_ra(3 downto 2) + 1;
			end if;
		end if;
		if(resetMem = '1')then
			queue_wa0SyncRegs <= (others => '0');
			queue_wa1SyncRegs <= (others => '0');
			queue_wap <= (others => '0');
		else
			queue_wa0SyncRegs <= queue_wa0SyncRegs(1 downto 0) & queue_wa(0);
			queue_wa1SyncRegs <= queue_wa1SyncRegs(1 downto 0) & queue_wa(1);
			queue_wap(1) <= queue_wa1SyncRegs(2);
			queue_wap(0) <= queue_wa0SyncRegs(2);
			if(queue_wap(1) = '1' and queue_wa1SyncRegs(2) = '0')then
				queue_wap(3 downto 2) <= queue_wap(3 downto 2) + 1;
			end if;
		end if;
		if(resetMem = '1' or (rd_cnt = "00001" and app_en_accept = '1'))then
			app_rqst_i <= '0';
		elsif(queue_do_vld = '1')then
			app_rqst_i <= '1';
		end if;
		if(app_rqst_i = '0')then
			app_addr_i <= '0' & queue_do(23 downto 0) & "000";
			rd_cnt <= queue_do(28 downto 24); -- rd_cnt is number of read in unit of 256 bit word(equivalent to eight 32-bit word)
		elsif(app_en_accept = '1')then
			app_addr_i(26 downto 3) <= app_addr_i(26 downto 3) + 1;
			rd_cnt <= rd_cnt - 1;
		end if;
		if(resetMem = '1')then
			buf_wa <= (others => '0');
		elsif(app_rd_data_valid = '1')then
			buf_wa(1) <= buf_wa(0);
			buf_wa(0) <= not buf_wa(1);
			if(buf_wa(1 downto 0) = "10")then
				buf_wa(8 downto 2) <= buf_wa(8 downto 2) + 1;
			end if;
		end if;
	end if;
end process;
g_rbuf : for i in 0 to 3 generate
	i_rbuf : BRAM_SDP_MACRO
   generic map (
      BRAM_SIZE => "36Kb", -- Target BRAM, "18Kb" or "36Kb" 
      DEVICE => "7SERIES", -- Target device: "VIRTEX5", "VIRTEX6", "7SERIES", "SPARTAN6" 
      WRITE_WIDTH => 64,    -- Valid values are 1-72 (37-72 only valid when BRAM_SIZE="36Kb")
      READ_WIDTH => 64,     -- Valid values are 1-72 (37-72 only valid when BRAM_SIZE="36Kb")
      DO_REG => 1, -- Optional output register (0 or 1)
      SIM_COLLISION_CHECK => "NONE", -- Collision check enable "ALL", "WARNING_ONLY", 
                                    -- "GENERATE_X_ONLY" or "NONE"       
      WRITE_MODE => "WRITE_FIRST", -- Specify "READ_FIRST" for same clock or synchronous clocks
                                   --  Specify "WRITE_FIRST for asynchrononous clocks on ports
      INIT => X"000000000000000000") --  Initial values on output port
   port map (
      DO => buf_do(i*64+63 downto i*64),         -- Output read data port, width defined by READ_WIDTH parameter
      DI => app_rd_data(i*64+63 downto i*64),         -- Input write data port, width defined by WRITE_WIDTH parameter
      RDADDR => buf_ra, -- Input read address, width defined by read port depth
      RDCLK => sysclk,   -- 1-bit input read clock
      RDEN => '1',     -- 1-bit input read port enable
      REGCE => ec_buf_do,   -- 1-bit input read output register enable
      RST => '0',       -- 1-bit input reset 
      WE => x"ff",         -- Input write enable, width defined by write port depth
      WRADDR => buf_wa, -- Input write address, width defined by write port depth
      WRCLK => memclk,   -- 1-bit input write clock
      WREN => app_rd_data_valid      -- 1-bit input write port enable
   );
end generate;
ec_buf_do <= '1' when resetSys = '0' and ec_buf_ra = '1' and (buf_do_vld = '0' or s_demux = "111") else '0';
end Behavioral;

